# ci.py - functions for handling Ecuadorian personal identity codes
# coding: utf-8
#
# Copyright (C) 2014 Jonathan Finlay
# Copyright (C) 2014-2017 Arthur de Jong
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301 USA

"""CI (Cédula de identidad, Ecuadorian personal identity code).

The CI is a 10 digit number used to identify Ecuadorian citizens.

>>> validate('171430710-3')
'1714307103'
>>> validate('1714307104')  # invalid check digit
Traceback (most recent call last):
    ...
InvalidChecksum: ...
>>> validate('171430710')  # digit missing
Traceback (most recent call last):
    ...
InvalidLength: ...
"""

from stdnum.exceptions import *
from stdnum.util import clean, isdigits


def compact(number):
    """Convert the number to the minimal representation. This strips the
    number of any valid separators and removes surrounding whitespace."""
    return clean(number, ' -').upper().strip()


def _checksum(number):
    """Calculate a checksum over the number."""
    fold = lambda x: x - 9 if x > 9 else x
    return sum(fold((2, 1)[i % 2] * int(n))
               for i, n in enumerate(number)) % 10


def validate(number):
    """Check if the number provided is a valid CI number. This checks the
    length, formatting and check digit."""
    number = compact(number)
    if len(number) != 10:
        raise InvalidLength()
    if not isdigits(number):
        raise InvalidFormat()
    if (number[:2] < '01' or number[:2] > '24') and (number[:2] not in ('30', '50')):
        raise InvalidComponent()  # invalid province code
    if number[2] > '6':
        raise InvalidComponent()  # third digit wrong
    if _checksum(number) != 0:
        raise InvalidChecksum()
    return number


def is_valid(number):
    """Check if the number provided is a valid CI number. This checks the
    length, formatting and check digit."""
    try:
        return bool(validate(number))
    except ValidationError:
        return False



from actions.delivery.delivery_address import request_and_save_delivery_address
from actions.delivery.get_delivery_fee_enabled_status import is_delivery_fee_enabled
from get_store_status import get_remote_order_status
from delivery_manual_order import create_delivery_manual_order_json



# ruc.py - functions for handling Ecuadorian fiscal numbers
# coding: utf-8
#
# Copyright (C) 2014 Jonathan Finlay
# Copyright (C) 2014-2021 Arthur de Jong
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301 USA

"""RUC (Registro Único de Contribuyentes, Ecuadorian company tax number).

The RUC is a tax identification number for legal entities. It has 13 digits
where the third digit is a number denoting the type of entity.

>>> validate('1792060346-001')
'1792060346001'
>>> validate('1763154690001')  # invalid check digit
Traceback (most recent call last):
    ...
InvalidChecksum: ...
>>> validate('179206034601')  # too short
Traceback (most recent call last):
    ...
InvalidLength: ...
"""

from stdnum.ec import ci
from stdnum.exceptions import *
from stdnum.util import isdigits


__all__ = ['compact', 'validate', 'is_valid']


# use the same compact function as CI
compact = ci.compact


def _checksum(number, weights):
    """Calculate a checksum over the number given the weights."""
    return sum(w * int(n) for w, n in zip(weights, number)) % 11


def _validate_natural(number):
    """Check if the number is a valid natural RUC (CI plus establishment)."""
    if number[-3:] == '000':
        raise InvalidComponent()  # establishment number wrong
    ci.validate(number[:10])
    return number


def _validate_public(number):
    """Check if the number is a valid public RUC."""
    if number[-4:] == '0000':
        raise InvalidComponent()  # establishment number wrong
    if _checksum(number[:9], (3, 2, 7, 6, 5, 4, 3, 2, 1)) != 0:
        raise InvalidChecksum()
    return number


def _validate_juridical(number):
    """Check if the number is a valid juridical RUC."""
    if number[-3:] == '000':
        raise InvalidComponent()  # establishment number wrong
    if _checksum(number[:10], (4, 3, 2, 7, 6, 5, 4, 3, 2, 1)) != 0:
        raise InvalidChecksum()
    return number


def validate(number):
    """Check if the number provided is a valid RUC number. This checks the
    length, formatting, check digit and check sum."""
    number = compact(number)
    if len(number) != 13:
        raise InvalidLength()
    if not isdigits(number):
        raise InvalidFormat()
    if (number[:2] < '01' or number[:2] > '24') and (number[:2] not in ('30', '50')):
        raise InvalidComponent()  # invalid province code
    if number[2] < '6':
        # 0..5 = natural RUC: CI plus establishment number
        _validate_natural(number)
    elif number[2] == '6':
        # 6 = public RUC (or natural RUC)
        try:
            _validate_public(number)
        except ValidationError:
            _validate_natural(number)
    elif number[2] == '9':
        # 9 = juridical RUC
        _validate_juridical(number)
    else:
        raise InvalidComponent()  # third digit wrong
    return number


def is_valid(number):
    """Check if the number provided is a valid RUC number. This checks the
    length, formatting and check digit."""
    try:
        return bool(validate(number))
    except ValidationError:
        return False



from abc import ABCMeta, abstractmethod
from logging import Logger

from cfgtools import Group
from dialogmanager import DialogManager
from mbcontextmessagehandler import MbContextMessageBus
from messagebus import (
    Event,
)

from modelcommon.model import SitefResponse

from typing import (
    Any,
    Union, Optional
)

from model import (
    CancelPaymentOrderData,
    ConfirmFinishPaymentData,
    FinishPaymentData,
    PaymentResponse,
    StartPaymentData,
    TestConnectionData,
)
from util import Cache


class PaymentProcessor(object):
    __metaclass__ = ABCMeta

    def __init__(
        self,
        message_bus,        # type: MbContextMessageBus
        dialog_manager,     # type: DialogManager
        config,             # type: Group
        logger,             # type: Logger
        cache,              # type: Optional[Cache]
    ):
        # type: (...) -> None

        self.message_bus = message_bus
        self.dialog_manager = dialog_manager
        self.config = config
        self.logger = logger
        self.cache = cache

    @abstractmethod
    def connection_status(self, test_connection_data):
        # type: (TestConnectionData) -> bool
        raise NotImplementedError()

    @abstractmethod
    def add_payment(self, start_payment_data):
        # type: (StartPaymentData) -> PaymentResponse
        raise NotImplementedError()

    @abstractmethod
    def cancel_payment_order(
        self,
        cancel_payment_order_data,  # type: CancelPaymentOrderData
        additional_params="",       # type: Optional[str]
    ):
        # type: (...) -> Any[Union[SitefResponse, str]]

        raise NotImplementedError()

    @abstractmethod
    def cancel_payment(self):
        # type: () -> str
        raise NotImplementedError()

    @abstractmethod
    def finish_payment(
        self,
        finish_payment_data     # type: FinishPaymentData
    ):
        # type: (...) -> bool
        raise NotImplementedError()

    @abstractmethod
    def confirm_finish_payment(
        self,
        confirm_finish_payment_data     # type: ConfirmFinishPaymentData
    ):
        # type: (...) -> None
        raise NotImplementedError()

    @abstractmethod
    def get_name(self):
        # type: () -> str
        raise NotImplementedError()

    @abstractmethod
    def get_document(
        self,
        pos_id      # type: str
    ):
        # type: (...) -> str

        raise NotImplementedError()

    def stop(self):
        # type: () -> None
        pass

    def get_qr_code_config(self):
        # type: () -> None
        pass

    @abstractmethod
    def get_last_receipt(
        self,
        eft_processor,      # type: Optional[str]
    ):
        # type: (...) -> str

        raise NotImplementedError()

    def register_eft_processor(
        self,
        pos_id,             # type: int
        processor_name      # type: str
    ):
        # type: (...) -> None

        event_data = str(processor_name)
        event_subject = "POS{}".format(pos_id)
        self.logger.debug("register_eft_processor: {}".format(event_data))

        event_xml = """<Event subject="{0}" type="{1}"><{2}>{3}</{2}></Event>"""
        event_xml = event_xml.format(event_subject, "REGISTER_EFT_PROCESSOR", "REGISTER_EFT_PROCESSOR", event_data)

        self.message_bus.publish_event(
            Event(
                subject=event_subject,
                event_type="REGISTER_EFT_PROCESSOR",
                data=event_xml,
            )
        )



"""
A pretty lame implementation of a memoryview object for Python 2.6.
"""

from collections import Iterable
from numbers import Integral
import string

from future.utils import istext, isbytes, PY3, with_metaclass
from future.types import no, issubset


# class BaseNewBytes(type):
#     def __instancecheck__(cls, instance):
#         return isinstance(instance, _builtin_bytes)


class newmemoryview(object):   # with_metaclass(BaseNewBytes, _builtin_bytes)):
    """
    A pretty lame backport of the Python 2.7 and Python 3.x
    memoryviewview object to Py2.6.
    """
    def __init__(self, obj):
        return obj


__all__ = ['newmemoryview']



import random
import time

import sysactions
from tablemgrapi import get_pos_ts
from xml.etree import cElementTree as eTree

large_data = None


@sysactions.action
def perform_network_test(
    pos_id,
):
    # type: (...) -> str

    global large_data
    start_time = time.time()
    if large_data is None:
        large_data = ""
        for i in range(1, 1 * 1024 * 1024):
            large_data += "b"

    model = sysactions.get_model(pos_id)
    if model is not None:
        pos_ot = sysactions.get_posot(model)

        current_order = sysactions.get_current_order(model)
        if current_order is not None:
            order_id = current_order.get("orderId")
            if order_id is not None:
                order_id = int(order_id)
                rand_order_id = random.randint(1, order_id)

                pos_ot.getOrderCustomProperties(orderid=str(rand_order_id))

        pos_ts = get_pos_ts(model)
        store_picture = pos_ts.storePicture(force_update=True)
        store_picture_xml = eTree.XML(store_picture)
        opened_table = store_picture_xml.find("Table[@status='3']")
        if opened_table is not None:
            table_id = opened_table.attrib["id"]
            pos_ts.getTablePicture(table_id)
    end_time = time.time()
    process_time = int((end_time - start_time) * 1000)
    return "{}-{}".format(process_time, large_data)



import json

from messagebus import AckMessage

from ._CommonProcessor import _CommonProcessor


class GetQRCodeConfigProcessor(_CommonProcessor):
    def parse_data(
        self,           # type: GetQRCodeConfigProcessor
        data,           # type: Any
        event_type,     # type: Any
    ):
        # type: (...) -> Any

        return data

    def call_business(
        self,   # type: GetQRCodeConfigProcessor
        _,      # type: Any
    ):
        # type: (...) -> Dict[str, str]

        return self.payment_router.get_qr_code_config()

    def format_response(
        self,           # type: GetQRCodeConfigProcessor
        message_bus,    # type: MessageBus
        message,        # type: Message
        event,          # type: Event
        input_obj,      # type: Any
        result,         # type: Dict[str, str]
    ):
        # type: (...) -> None

        message_bus.reply_message(
            message=message,
            reply_message=AckMessage(json.dumps(result).encode("utf-8"))
        )



from production.box import FilterOrderIfNotAllItemsHaveTagBox
from production.treebuilder import BoxConfiguration

from ._BoxBuilder import BoxBuilder


class FilterOrderIfNotAllItemsHaveTagBoxBuilder(BoxBuilder):
    def __init__(self, loggers):
        super(FilterOrderIfNotAllItemsHaveTagBoxBuilder, self).__init__(loggers)

    def build(self, config):
        # type: (BoxConfiguration) -> FilterOrderIfNotAllItemsHaveTagBox
        return FilterOrderIfNotAllItemsHaveTagBox(config.name,
                                                  config.sons,
                                                  self.get_extra(config, "AllowedTags", []),
                                                  self.get_extra(config, "ForbiddenTags", []),
                                                  self.get_extra(config, "RemoveOrder", "true").lower() == "true",
                                                  self.get_logger(config))



# ird.py - functions for handling New Zealand IRD numbers
# coding: utf-8
#
# Copyright (C) 2019 Leandro Regueiro
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301 USA

"""IRD number (New Zealand Inland Revenue Department (Te Tari Tāke) number).

The IRD number is used by the New Zealand Inland Revenue Department (Te Tari
Tāke in Māori) to identify businesses and individuals for tax purposes. The
number consists of 8 or 9 digits where the last digit is a check digit.

More information:

* https://www.ird.govt.nz/
* https://www.ird.govt.nz/-/media/Project/IR/PDF/2020RWTNRWTSpecificationDocumentv10.pdf
* https://www.oecd.org/tax/automatic-exchange/crs-implementation-and-assistance/tax-identification-numbers/New%20Zealand-TIN.pdf

>>> validate('4909185-0')
'49091850'
>>> validate('NZ 49-098-576')
'49098576'
>>> validate('136410133')
Traceback (most recent call last):
    ...
InvalidChecksum: ...
>>> validate('9125568')
Traceback (most recent call last):
    ...
InvalidLength: ...
>>> format('49098576')
'49-098-576'
"""  # noqa: E501

from stdnum.exceptions import *
from stdnum.util import clean, isdigits


def compact(number):
    """Convert the number to the minimal representation."""
    number = clean(number, ' -').upper().strip()
    if number.startswith('NZ'):
        return number[2:]
    return number


def calc_check_digit(number):
    """Calculate the check digit.

    The number passed should not have the check digit included.
    """
    primary_weights = (3, 2, 7, 6, 5, 4, 3, 2)
    secondary_weights = (7, 4, 3, 2, 5, 2, 7, 6)
    # pad with leading zeros
    number = number.zfill(8)
    s = -sum(w * int(n) for w, n in zip(primary_weights, number)) % 11
    if s != 10:
        return str(s)
    s = -sum(w * int(n) for w, n in zip(secondary_weights, number)) % 11
    return str(s)


def validate(number):
    """Check if the number is a valid IRD number."""
    number = compact(number)
    if len(number) not in (8, 9):
        raise InvalidLength()
    if not isdigits(number):
        raise InvalidFormat()
    if not 10000000 < int(number) < 150000000:
        raise InvalidComponent()
    if number[-1] != calc_check_digit(number[:-1]):
        raise InvalidChecksum()
    return number


def is_valid(number):
    """Check if the number is a valid IRD number."""
    try:
        return bool(validate(number))
    except ValidationError:
        return False


def format(number):
    """Reformat the number to the standard presentation format."""
    number = compact(number)
    return '-'.join([number[:-6], number[-6:-3], number[-3:]])



# edrpou.py - functions for handling Ukrainian EDRPOU numbers
# coding: utf-8
#
# Copyright (C) 2020 Leandro Regueiro
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301 USA

"""ЄДРПОУ, EDRPOU (Identifier for enterprises and organizations in Ukraine).

The ЄДРПОУ (Єдиного державного реєстру підприємств та організацій України,
Unified State Register of Enterprises and Organizations of Ukraine) is a
unique identification number of a legal entities in Ukraine. Th number
consists of 8 digits, the last being a check digit.

More information:

* https://uk.wikipedia.org/wiki/Код_ЄДРПОУ
* https://1cinfo.com.ua/Articles/Proverka_koda_po_EDRPOU.aspx

>>> validate('32855961')
'32855961'
>>> validate('32855968')
Traceback (most recent call last):
    ...
InvalidChecksum: ...
>>> validate('12345')
Traceback (most recent call last):
    ...
InvalidLength: ...
>>> format(' 32855961 ')
'32855961'
"""

from stdnum.exceptions import *
from stdnum.util import clean, isdigits


def compact(number):
    """Convert the number to the minimal representation."""
    return clean(number, ' ').strip()


def calc_check_digit(number):
    """Calculate the check digit for number."""
    weights = (1, 2, 3, 4, 5, 6, 7)
    if number[0] in '345':
        weights = (7, 1, 2, 3, 4, 5, 6)
    total = sum(w * int(n) for w, n in zip(weights, number))
    if total % 11 < 10:
        return str(total % 11)
    # Calculate again with other weights
    weights = tuple(w + 2 for w in weights)
    total = sum(w * int(n) for w, n in zip(weights, number))
    return str(total % 11)


def validate(number):
    """Check if the number is a valid Ukraine EDRPOU (ЄДРПОУ) number.

    This checks the length, formatting and check digit.
    """
    number = compact(number)
    if len(number) != 8:
        raise InvalidLength()
    if not isdigits(number):
        raise InvalidFormat()
    if number[-1] != calc_check_digit(number):
        raise InvalidChecksum()
    return number


def is_valid(number):
    """Check if the number is a valid Ukraine EDRPOU (ЄДРПОУ) number."""
    try:
        return bool(validate(number))
    except ValidationError:
        return False


def format(number):
    """Reformat the number to the standard presentation format."""
    return compact(number)



# utr.py - functions for handling English UTRs
#
# Copyright (C) 2020 Holvi Payment Services
# Copyright (C) 2020 Arthur de Jong
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301 USA

"""UTR (United Kingdom Unique Taxpayer Reference).

A UTR (unique taxpayer reference) is a 10 digit number used to identify UK
taxpayers who have to submit a tax return.

More information:

* https://www.oecd.org/tax/automatic-exchange/crs-implementation-and-assistance/tax-identification-numbers/UK-TIN.pdf

>>> validate('1955839661')
'1955839661'
>>> validate('2955839661')
Traceback (most recent call last):
    ...
InvalidChecksum: ..
"""

from stdnum.exceptions import *
from stdnum.util import clean, isdigits


def compact(number):
    """Convert the number to the minimal representation. This strips the
    number of any valid separators and removes surrounding whitespace."""
    return clean(number, ' ').upper().strip().lstrip('K')


def calc_check_digit(number):
    """Calculate the check digit for the number. The passed number should not
    have the check digit (the first one) included."""
    weights = (6, 7, 8, 9, 10, 5, 4, 3, 2)
    return '21987654321'[sum(int(n) * w for n, w in zip(number, weights)) % 11]


def validate(number):
    """Check if the number is a valid UTR."""
    number = compact(number)
    if not isdigits(number):
        raise InvalidFormat()
    if not len(number) == 10:
        raise InvalidLength()
    if number[0] != calc_check_digit(number[1:]):
        raise InvalidChecksum()
    return number


def is_valid(number):
    """Check if the number is a valid UTR."""
    try:
        return bool(validate(number))
    except ValidationError:
        return False



import base64

from messagebus import MessageBus
from sysactions import get_model

from production.model import PickListOrderNotFoundException
from production.repository import ProductionRepository
from production.service import PrinterService
from production.util import parse_constants
from typing import (
    Dict,
    Optional,
)


class PickListOrdersInteractor(object):
    def __init__(
        self,
        message_bus,                # type: MessageBus
        production_repository,      # type: ProductionRepository
    ):
        # type: (...) -> None

        self.timeout = 10000 * 1000
        self.production_repository = production_repository
        self.printer_service = PrinterService(
            message_bus=message_bus,
            timeout=self.timeout,
        )

    def execute(
        self,
        obj,    # type: Dict
    ):
        # type: (...) -> Optional[Dict]

        pos_id = obj.get("posId")
        orders_list = obj.get("ordersList")
        pick_list_order_id = obj.get("selectedOption", None)

        if pick_list_order_id is not None:
            printer_name = obj.get("printerName", None)
            self._print(
                pos_id=pos_id,
                printer_name=printer_name,
                pick_list_order_id=int(pick_list_order_id),
            )
            return {}

        return self.production_repository.get_all_pick_list_orders(
            orders_list=orders_list,
        )

    def _print(
        self,
        pos_id,                 # type: str
        printer_name,           # type: str
        pick_list_order_id,     # type: str
    ):
        # type: (...) -> None

        result = self.production_repository.get_data_pick_list_order(
            pick_list_order_id=pick_list_order_id,
        )

        if not result:
            raise PickListOrderNotFoundException()

        pick_list_to_print = base64.b64decode(result)

        pick_list_to_print = parse_constants(
            pos_id=pos_id,
            printer_text=pick_list_to_print,
            is_reprint=True,
        )

        self.printer_service.send_to_printer(
            printer_name=printer_name,
            text=pick_list_to_print,
        )



import json
import logging

from msgbus import (
    FM_PARAM,
    MBException,
    TK_SYS_ACK,
)
from sysactions import send_message
from model.customexception import (
    ComponentNotFound,
    ErrorGettingSalesChallenge,
    ErrorOnInsertSalesChallenge,
    ErrorOnDeleteSalesChallenge,
    ErrorOnUpdateSalesChallenge,
)
from bustoken import TK_DAILY_GOALS_GET_SALES_CHALLENGE, TK_DAILY_GOALS_INSERT_SALES_CHALLENGE,\
    TK_DAILY_GOALS_DELETE_SALES_CHALLENGE, TK_DAILY_GOALS_UPDATE_SALES_CHALLENGE


class DailyGoalsInteractor(object):
    def __init__(self):
        # type: () -> None
        self.logger = logging.getLogger("PosActions")

    @staticmethod
    def get_sales_challenge(pos_id, quantity):
        # type: (int, int) -> json

        data = json.dumps(dict(posId=pos_id, quantity=quantity))

        try:
            response = send_message(
                dest_name="DailyGoals",
                token=TK_DAILY_GOALS_GET_SALES_CHALLENGE,
                format=FM_PARAM,
                data=data,
                timeout=30000000,
            )
            if response.token == TK_SYS_ACK:
                return json.loads(response.data)

            raise ErrorGettingSalesChallenge(response.data)

        except MBException as ex:
            if "SE_NOTFOUND" in ex.message:
                raise ComponentNotFound("DailyGoals")

            raise ErrorGettingSalesChallenge(ex)

    @staticmethod
    def insert_sales_challenge(sale_challenge):
        # type: (dict) -> bool

        data = json.dumps(sale_challenge)

        response = send_message("DailyGoals", TK_DAILY_GOALS_INSERT_SALES_CHALLENGE, FM_PARAM, data, timeout=30000000)
        if response.token == TK_SYS_ACK:
            return True

        raise ErrorOnInsertSalesChallenge(response.data)

    @staticmethod
    def update_sales_challenge(sale_challenge):
        # type: (dict) -> bool

        data = json.dumps(sale_challenge)

        response = send_message("DailyGoals", TK_DAILY_GOALS_UPDATE_SALES_CHALLENGE, FM_PARAM, data, timeout=30000000)
        if response.token == TK_SYS_ACK and response.data == '1':
            return True

        raise ErrorOnUpdateSalesChallenge(response.data)

    @staticmethod
    def remove_sales_challenge(sale_challenge_id):
        # type: (int) -> json

        data = json.dumps(dict(sale_challenge_id=sale_challenge_id))

        response = send_message("DailyGoals", TK_DAILY_GOALS_DELETE_SALES_CHALLENGE, FM_PARAM, data, timeout=30000000)
        if response.token == TK_SYS_ACK and response.data == '1':
            return json.loads(response.data)

        raise ErrorOnDeleteSalesChallenge(response.data)



# -*- coding: utf-8 -*-
import sysactions
from sysactions import check_operator_logged, StopAction

from msgbus import TK_POS_USERPAUSE, FM_PARAM, TK_SYS_ACK
from typing import Optional

from .. import logger


@sysactions.action
def pause_user(
    pos_id,                         # type: str
    pos_id_to_show_message=None,    # type: Optional[str]
):
    # type: (...) -> bool

    try:
        check_operator_logged(pos_id)
        pos_id_to_show_message = pos_id if pos_id_to_show_message is None else pos_id_to_show_message
        from table_actions import deselect_table
        deselected_success = deselect_table(
            pos_id=pos_id,
            do_raise=False
        )

        if not deselected_success:
            return False

        msg = sysactions.send_message(
            "POS{}".format(pos_id),
            TK_POS_USERPAUSE,
            FM_PARAM,
            pos_id,
            timeout=60 * 1000000
        )
        if msg.token != TK_SYS_ACK:
            error_code = msg.data.split('\0')[0]
            if error_code == "5003":  # POS BLOCKED BY TIME
                return True

            logger.error("Invalid response from PosCtrl while Pausing user: {} - {}".format(error_code, msg.data))
            _show_error_login_message(pos_id=pos_id_to_show_message)
            return False
    except StopAction:
        return False
    except (Exception, BaseException):
        logger.exception("Exception while pausing user on POS: {}".format(pos_id))
        _show_error_login_message(pos_id=pos_id_to_show_message)
        return False
    return True


def _show_error_login_message(
    pos_id,     # type: str
):
    sysactions.show_messagebox(
        posid=pos_id,
        message="$INVALID_USERNAME_OR_PASSWORD",
    )



import logging

from model.customexception import RestartComponentException

from msgbus import TK_CMP_TERM_NOW
from sysactions import send_message

logger = logging.getLogger("PosActions")


def restart_pos(
    pos_id,     # type: str
):
    # type: (...) -> None

    logger.info("Restarting POS {}".format(pos_id))
    component = "POS%d" % int(pos_id)
    response = send_message(
        dest_name=component,
        token=TK_CMP_TERM_NOW,
        timeout=600000000
    )
    if response.token != TK_CMP_TERM_NOW:
        raise RestartComponentException(
            component=component,
            token=response.token,
            data=response.data,
        )

    logger.info("POS {} successfully restarted".format(pos_id))



import logging

from typing import (
    Dict,
    Optional,
    Tuple,
)

from devicemanager.model import (
    RequesterType,
)
from devicemanager.service import (
    DeviceStatusService,
    UsedServiceCache,
)
from .dto import GetUsedServiceRequest

logger = logging.getLogger(__name__)


class GetUsedServiceInteractor(object):
    def __init__(
        self,
        device_status_service,  # type: DeviceStatusService
        used_service_cache,     # type: UsedServiceCache
        sector_config,          # type: Dict[str, Dict[str, Tuple[str, Dict[int, int]]]]
        brand_config,           # type: Dict[str, Dict[str, str]]
    ):
        self.device_status_service = device_status_service
        self.used_service_cache = used_service_cache
        self.sector_config = sector_config
        self.brand_config = brand_config

    def get_used_service(
        self,       # type: GetUsedServiceInteractor
        request,    # type: GetUsedServiceRequest
    ):
        # type: (...) -> Optional[str]

        if request.device_type == "printer":
            logger.info("Getting printer device")
            return self._get_printer(request=request)

        return self._get_used_service_from_pos_model(request=request)

    def _get_printer(
        self,       # type: GetUsedServiceInteractor
        request,    # type: GetUsedServiceRequest
    ):
        # type: (...) -> Optional[str]

        preselected_printer = self._get_preselected_printer(request=request)
        if preselected_printer:
            return preselected_printer

        return self._get_requester_printer(request=request)

    def _get_preselected_printer(
        self,       # type: GetUsedServiceInteractor
        request,    # type: GetUsedServiceRequest
    ):
        # type: (...) -> Optional[str]

        brand_printer = self._get_brand_printer(
            brand=request.brand,
            brand_config=self.brand_config,
        )
        if brand_printer:
            logger.info("Retrieved brand printer: {}".format(brand_printer))
            return brand_printer

        sector_printer = self._get_sector_printer(
            sector=request.sector,
            sector_config=self.sector_config,
            pos_id=request.pos_id,
        )

        logger.info("Retrieved sector printer: {}".format(brand_printer))
        return sector_printer

    @staticmethod
    def _validate_key_config(
        key,            # type: str
        key_config,     # type: Dict
    ):
        # type: (...) -> Optional[str]

        if key is None:
            return None

        key = key.upper()
        if key not in key_config or u"PRINTER" not in key_config[key]:
            return None

        return key

    def _get_brand_printer(
        self,           # type: GetUsedServiceInteractor
        brand,          # type: str
        brand_config,   # type: Dict[str, Dict[str, str]]
    ):
        # type: (...) -> Optional[str]

        valid_brand = self._validate_key_config(
            key=brand,
            key_config=brand_config,
        )

        if not valid_brand:
            return None

        device_name = brand_config[valid_brand]["PRINTER"]

        if self.device_status_service.is_device_enabled(device_name=device_name):
            return device_name

        return None

    def _get_sector_printer(
        self,           # type: GetUsedServiceInteractor
        sector,         # type: str
        sector_config,  # type: Dict[str, Dict[str, Tuple[str, Dict[int, int]]]]
        pos_id,         # type: int
    ):
        # type: (...) -> Optional[str]

        valid_sector = self._validate_key_config(
            key=sector,
            key_config=sector_config,
        )

        if not valid_sector:
            return None

        device_name, excluded_pos = sector_config[valid_sector]["PRINTER"]
        if pos_id in excluded_pos:
            return None

        if self.device_status_service.is_device_enabled(device_name=device_name):
            return device_name

        return None

    def _get_requester_printer(
        self,       # type: GetUsedServiceInteractor
        request,    # type: GetUsedServiceRequest
    ):
        # type: (...) -> Optional[str]

        requester_type = RequesterType.POS if request.pos_id is not None else RequesterType.KDS
        requester_id = request.pos_id if request.pos_id is not None else request.kds_id
        logger.info("Getting printer from {}{}".format(requester_type, requester_id))
        return self.device_status_service.get_enabled_device(
            requester_type=requester_type,
            requester_id=requester_id
        )

    def _get_used_service_from_pos_model(
        self,       # type: GetUsedServiceInteractor
        request,    # type: GetUsedServiceRequest
    ):
        # type: (...) -> Optional[str]

        return self.used_service_cache.get_cached_object(
            device_type=request.device_type,
            pos_id=request.pos_id,
        )



import json

from sysactions import (
    StopAction,
    action,
    get_model,
    show_any_dialog,
    show_messagebox,
    translate_message,
)

from mw_helper import show_filtered_list_box_dialog
from pos_model import TableOperationMode
from typing import (
    Any,
    Dict,
    List,
    Tuple,
    Union,
)

from .. import (
    logger,
    mb_context,
)
from actions.models.user_levels import UserLevels
from operator_by_sector import (
    get_sectors,
    get_operator_sector_info,
)
from userhelper import UserHelper

ALL_OPERATORS = []
ALL_SECTORS = "$ALL_SECTORS"


def _select_sector(
    pos_id,     # type: str
):
    # type: (...) -> Tuple[str,str]

    sectors = get_sectors(pos_id=pos_id)

    selected_index = show_filtered_list_box_dialog(
        pos_id=pos_id,
        options=sectors,
        title="$SELECT_ONE_SECTOR",
        mask="NOFILTER",
    )
    if selected_index is None:
        raise StopAction()

    selected_sector = sectors[selected_index]
    if selected_sector.startswith('$'):
        model = get_model(posid=pos_id)
        selected_sector = translate_message(
            model=model,
            message_key=str(selected_sector[1:]),
        )

    return selected_sector, sectors[selected_index]


def _get_all_operators():
    # type: (...) -> List[Dict]

    global ALL_OPERATORS
    if ALL_OPERATORS:
        return ALL_OPERATORS

    with UserHelper(mb_context) as user_helper:
        all_operators = user_helper.get_all_users()
        ALL_OPERATORS = []
        for operator_id in sorted(all_operators, key=lambda x: int(x)):
            operator = all_operators.get(operator_id)
            if int(operator[1]) < UserLevels.MANAGER.value:
                user_info = dict(
                    id=int(operator_id),
                    name=operator[0],
                )
                ALL_OPERATORS.append(user_info)

        return ALL_OPERATORS


def _filter_operators(
    all_operators,          # type: List[Dict[str, Any[Union[str, int]]]]
    sector_operators_ids,   # type: List[int]
):
    # type: (...) -> List[Dict[str, Any[Union[str, int]]]]

    sector_operators = []
    for operator in all_operators:
        if operator.get("id") in sector_operators_ids:
            sector_operators.append(operator)
    return sector_operators


def _get_sector_operators(
    pos_id,     # type: str
    sector,     # type: str
):
    # type: (...) -> List[Dict[str, Any[Union[str, int]]]]

    all_sector_operators = get_operator_sector_info(
        pos_id=pos_id,
        repeat_in_sub_sector=False,
    )
    if sector != ALL_SECTORS and sector not in all_sector_operators:
        return []

    all_operators = _get_all_operators()
    if sector != ALL_SECTORS:
        return _filter_operators(
            all_operators=all_operators,
            sector_operators_ids=all_sector_operators.get(sector),
        )

    all_sectors = get_sectors(pos_id=pos_id)
    sectors_name = all_sector_operators.keys()
    if len(all_sectors) - 1 != len(sectors_name):
        return []

    current_operators = all_sector_operators.get(sectors_name[0])
    if len(sectors_name) > 1:
        for sector_name in sectors_name[1:]:
            current_operators = list(set(current_operators) & set(all_sector_operators.get(sector_name)))

    return _filter_operators(
        all_operators=all_operators,
        sector_operators_ids=current_operators,
    )


def _get_available_operators(
    operators_to_ignore,    # type: List[int]
):
    # type: (...) -> List[Dict[str, Any[Union[str, int]]]]

    operators = []
    all_operators = _get_all_operators()
    for operator in all_operators:
        if operator.get("id") not in operators_to_ignore:
            operators.append(operator)

    return operators


@action
def open_service_sector(
    pos_id,     # type: str
):
    # type: (...) -> None

    from table_actions import table_operation_mode
    if table_operation_mode not in [TableOperationMode.INDIVIDUAL_SECTOR, TableOperationMode.SHARED_SECTOR]:
        show_messagebox(
            posid=pos_id,
            message="$NOT_VALID_TABLE_OPERATION_MODE",
            title="$INFORMATION",
        )
        raise StopAction()

    try:
        selected_sector, original_selected_sector = _select_sector(pos_id=pos_id)
        sector_operators = _get_sector_operators(
            pos_id=pos_id,
            sector=original_selected_sector,
        )
        available_operators = _get_available_operators(
            operators_to_ignore=list(map(lambda s: s.get("id"), sector_operators)),
        )
    except Exception as e:
        logger.exception('[rupture_items] Exception {}'.format(e))
        show_messagebox(
            posid=pos_id,
            message="$ERROR_GETTING_DATA",
            title="$INFORMATION",
            buttons="$EXIT",
        )
        raise StopAction()

    dialog_contents = {
        "type": "service_sector",
        "config": {
            "selectedSector": original_selected_sector,
            "leftTitle": "$OPERATORS",
            "rightTitle": selected_sector,
            "confirmTitle": "$CONFIRM_UPDATE_OPERATORS_BY_SECTOR",
            "confirmLeftTitle": "$LEAVE_THE_SECTOR|{0}".format(selected_sector),
            "confirmRightTitle": "$ENTER_THE_SECTOR|{0}".format(selected_sector)
        },
        "availableItems": available_operators,
        "selectedItems": sector_operators
    }

    info_text = json.dumps(dialog_contents)
    show_any_dialog(
        posid=pos_id,
        dlgtype="enableItemsDialog",
        message="",
        title="$MANAGE_SERVICE_SECTOR",
        infotext=info_text,
        buttons="",
        icon="",
        timeout=6000000,
        mask="",
        defvalue="",
        minvalue="",
        maxvalue="",
        contents="",
        asynch=False,
    )



import json
import os

from copy import deepcopy
from datetime import datetime
from threading import Lock
from os.path import isfile, join

from systools import sys_parsefilepath

import pytz
from application.model import UpdateType, MediaUpdateData, UpdateMediaJson
from application.model.configuration import Configurations
from application.model.customexception import ErrorRetrievingPendingUpdate, ErrorApplyingUpdate, \
    HasNoProductsImagesToDownload, NoUpdateFound, NotExpectedException, DisabledFunctionality, \
    ErrorGettingTotemConfig, NotAllImagesDownload
from application.repository.api import MediaUpdaterAPIRepository
from application.repository.pos import MediaUpdaterPOSRepository
from application.service import UpdaterService
from iso8601 import iso8601
from requests import Response
from typing import (
    Any,
    Dict,
    List,
    Optional,
    Union,
)
from application.util import LoggerUtil

logger = LoggerUtil.get_logger_name(__name__)

allowed_video_extensions = ['mp4', 'webm', 'ogg']


class MediaUpdaterService(object):
    def __init__(
        self,
        configs,            # type: Configurations
        working_on_update,  # type: Lock
        api_repository,     # type: MediaUpdaterAPIRepository
        pos_repository,     # type: MediaUpdaterPOSRepository
        updater_service,    # type: UpdaterService
        enabled,            # type: bool
    ):
        # type: (...) -> None

        self.logger = logger
        self.configs = configs
        self.update_type = UpdateType.media
        self.local_configs = configs.updaters[self.update_type.name]
        self.working_on_update = working_on_update
        self.api_repository = api_repository
        self.pos_repository = pos_repository
        self.updater_service = updater_service
        self.enabled = enabled

        self.images_path = self.local_configs.images_directory
        if not os.path.exists(self.images_path):
            os.makedirs(self.images_path)

        data_path = sys_parsefilepath("{HVDATADIR}")

        self.control_json_path = os.path.join(self.images_path, "imagesHashes.json")
        self.control_json_totem_path = os.path.join(self.images_path, "../static/totemConfig.json")

        self.totem_config_json_file_name = "totemConfig.json"
        self.totem_config_original_path = "../src/internalorder/static"
        self.totem_config_destination_path = os.path.join(data_path, "bundles", "internalorder")

    def update_media(
        self,
        force_update,   # type: bool
    ):
        # type: (...) -> None

        if not self.enabled:
            raise DisabledFunctionality

        try:
            self.logger.info("Trying to update media...")
            some_media_downloaded = False
            if force_update or self.pos_repository.has_pending_update_to_apply(UpdateType.banners.value):
                totem_configs = self._get_totem_configs()
                some_media_downloaded |= self._download_totem_media(totem_configs)
                self.update_totem_config_json(totem_configs)
            if force_update or self.pos_repository.has_pending_update_to_apply():
                some_media_downloaded |= self._download_media()

            self.logger.info("Media update fully concluded!")
            if not some_media_downloaded:
                raise NoUpdateFound()
        except NoUpdateFound as _:
            self.logger.info("No pending updates found")
            raise
        except HasNoProductsImagesToDownload as _:
            self.logger.info("Has not products to download images")
            raise
        except ErrorRetrievingPendingUpdate as _:
            self.logger.exception("Error retrieving pending update")
            raise
        except ErrorApplyingUpdate as _:
            self.logger.error("Error doing update")
            raise
        except (NotExpectedException, Exception) as _:
            self.logger.exception("Not expected exception on media update")
            raise

    def _download_media(self):
        # type: () -> bool

        self.logger.info("Trying to update media...")

        json_images_hash = self._get_images_hashes_from_json()
        catalog_db = self.pos_repository.get_images_hashes_from_database()
        navigation_db = self.pos_repository.get_images_hashes_from_navigation()
        try:
            catalog = json_images_hash.catalog
            navigation = json_images_hash.navigation
            images_to_download = self._filter_images_to_download(catalog_db, catalog)
            images_to_download.update(self._filter_images_to_download(navigation_db, navigation))

            if not images_to_download:
                self.pos_repository.insert_new_media_update()
                return False

            images_to_download = self._format_images_to_update_data(images_to_download, navigation_db)
            self._get_images_permissions(images_to_download)
            self._download_and_store_images(images_to_download)
            json_images_hash.catalog = self._update_images_hashes_json(catalog_db, catalog, images_to_download)
            json_images_hash.navigation = self._update_images_hashes_json(navigation_db, navigation, images_to_download)
            not_downloaded_images = self._validate_all_images_was_downloaded(images_to_download)
            self._save_file(json_images_hash)
            self.pos_repository.insert_new_media_update()

            if len(not_downloaded_images) > 0:
                self.logger.info("Not all images was downloaded: {}".format(",".join(not_downloaded_images)))
                raise NotAllImagesDownload()
        finally:
            self._remove_old_images_catalog_and_navigation(json_images_hash)

        return True

    def update_totem_config_json(
        self,
        totem_configs,  # type: Dict
    ):
        # type: (...) -> None

        if not os.path.exists(self.totem_config_destination_path):
            return

        destination_file = self._create_totem_config_file_if_not_exists()
        self._update_totem_config_json(destination_file, totem_configs)

    @staticmethod
    def _get_custom_images_totem_configs(
        totem_configs,    # type: Dict
    ):
        # type: (...) -> List[dict]

        custom_images = []
        if "customImages" in totem_configs:
            for custom_image in totem_configs.get("customImages"):
                key = custom_image.keys()[0]
                img_path = custom_image.get(key).get("imagePath")
                if "sui" not in img_path:
                    custom_image[key]["imagePath"] = "sui/images/{}".format(img_path)
                custom_images.append(custom_image)
        return custom_images

    @staticmethod
    def _parse_media_to_totem_config(
        totem_configs,                         # type: Dict[str, Dict[str, str]]
        key,                                   # type: str
        image_path_format="/sui/images/{}",    # type: Optional[str]
    ):
        # type: (...) -> Dict[str, str]

        media_configs = {}
        if totem_configs.get(key):
            for media in totem_configs.get(key, []):
                media_id = media.get("imageId", "")
                media_name = media.get("name", "")
                media_configs[media_id] = image_path_format.format(media_name)

        return media_configs

    def _update_totem_config_json(
        self,
        destination_file,   # type: str
        totem_configs,      # type: Dict
    ):
        # type: (...) -> None

        image_path_format = "/sui/images/{}"

        with open(destination_file, "r") as totem_config_file:
            new_totem_configs = json.load(totem_config_file)

            if totem_configs.get("style"):
                new_totem_configs["style"] = totem_configs.get("style")

            if totem_configs.get("banner"):
                new_totem_configs["banner"] = totem_configs.get("banner")

                banners = totem_configs.get("banner")
                for banner in banners["images"]:
                    banner["name"] = image_path_format.format(banner["name"])

            if totem_configs.get("payments"):
                new_totem_configs["payments"] = totem_configs.get("payments")

                payments = totem_configs.get("payments")
                for payment_id in payments.keys():
                    payments[payment_id]["name"] = image_path_format.format(payments[payment_id]["name"])

            new_totem_configs["images"].update(
                self._parse_media_to_totem_config(
                    totem_configs=totem_configs,
                    key="images",
                )
            )

            new_totem_configs['customImages'] = self._get_custom_images_totem_configs(
                totem_configs=totem_configs,
            )

            new_totem_configs["videos"] = {}
            if totem_configs.get('videos', []):
                new_totem_configs["videos"].update(
                    self._parse_media_to_totem_config(
                        totem_configs=totem_configs,
                        key="videos",
                    )
                )

        with open(destination_file, "w") as totem_config_file:
            json.dump(new_totem_configs, totem_config_file)

    def _create_totem_config_file_if_not_exists(
        self
    ):
        # type: (...) -> str

        destination_file = os.path.join(
            self.totem_config_destination_path,
            self.totem_config_json_file_name
        )
        valid_totem_config = False

        try:
            if os.path.exists(destination_file):
                with open(destination_file, "r") as totem_config_file:
                    json.load(totem_config_file)
                    valid_totem_config = True
        except ValueError as _:
            pass

        if not valid_totem_config:
            original_file = os.path.join(
                self.totem_config_original_path,
                self.totem_config_json_file_name
            )
            self.updater_service.copy_file(
                original_file_path=original_file,
                destination_file_path=destination_file,
            )

        return destination_file

    def _remove_old_images_catalog_and_navigation(self, images_hash):
        # type: (UpdateMediaJson) -> None

        try:
            prt_nav_images = [f for f in os.listdir(self.images_path) if self._is_product_or_nav(f)]
            prt_images = ["PRT" + product_code.zfill(8) for product_code in images_hash.catalog.keys()]
            nav_images = ["NAV" + product_code.zfill(8) for product_code in images_hash.navigation.keys()]

            for file_name in prt_nav_images:
                if file_name[:11] in prt_images or file_name[:11] in nav_images:
                    continue

                os.remove(os.path.join(self.images_path, file_name))

        except Exception as _:
            self.logger.exception("Error remove old images")

    def _is_product_or_nav(self, file_name):
        # type: (str) -> bool

        is_product = file_name.startswith("PRT")
        is_navigation = file_name.startswith("NAV")
        return isfile(join(self.images_path, file_name)) and (is_product or is_navigation)

    @staticmethod
    def _validate_all_images_was_downloaded(images_to_download):
        images_not_downloaded = []
        for image in images_to_download:
            if not image.downloaded:
                images_not_downloaded.append(image.product_code)

        return images_not_downloaded

    def _download_totem_media(
        self,
        totem_configs,  # type: Dict
    ):
        # type: (...) -> bool

        some_totem_media_downloaded = False
        try:
            images_hashes = self._get_images_hashes_from_json()
            some_totem_media_downloaded |= self._download_banner_images(
                totem_configs=totem_configs,
                images_hashes=images_hashes,
            )
            some_totem_media_downloaded |= self._download_totem_images(
                totem_configs=totem_configs,
                stored_images_hash=images_hashes,
            )
            self.pos_repository.insert_new_media_update(UpdateType.banners.value)
        except ErrorGettingTotemConfig as _:
            self.logger.warning("Error downloading totem media")
        except Exception as _:
            self.logger.exception("Error downloading totem media")

        return some_totem_media_downloaded

    def _download_banner_images(
        self,
        totem_configs,  # type: Dict
        images_hashes,  # type: UpdateMediaJson
    ):
        # type: (...) -> bool

        self.logger.info("Trying to update banner media...")

        some_banner_media_downloaded = False
        stored_banners = images_hashes.banners_totem.copy()
        banners_json = totem_configs.get("banner", "{}")
        images_to_download = self._get_medias_to_download(banners_json, stored_banners.values())
        self._update_from_remote_medias(
            totem_configs=totem_configs,
            stored_medias_hash=images_hashes,
            medias_to_download=images_to_download,
            is_banner=True,
        )

        if len(images_to_download) > 0:
            images_to_download = self._download_and_save(images_to_download)

            images_hash = self._get_medias_hashes_from_stored_json()
            banners_totem = self._remove_and_update_config(
                totem_configs=totem_configs,
                medias_to_download=images_to_download,
                medias_hash=images_hash,
                medias_hashes_from_json=stored_banners,
                is_banner=True,
            )
            images_hashes.banners_totem = banners_totem.get('images')
            self._save_file(images_hashes)
            some_banner_media_downloaded = True

        return some_banner_media_downloaded

    @staticmethod
    def _get_medias_to_delete(
        totem_configs,          # type: dict
        stored_image_hashes,    # type: list
        is_banner=False,        # type: Optional[bool]

    ):
        # type: (...) -> list

        totem_images = totem_configs if not is_banner else totem_configs.get('banner', {}) or {}

        medias_to_download = [img_data.get('hashImage') for img_data in totem_images.get('images', [])]
        medias_to_download += [video_data.get('hashImage') for video_data in totem_images.get('videos', [])]

        images_to_delete = []
        for img_hash in stored_image_hashes:
            if img_hash not in medias_to_download:
                images_to_delete.append(img_hash)

        return images_to_delete

    def _delete_medias_files(
        self,
        stored_hashes_json,     # type: dict
        hashes_to_delete,       # type: list

    ):
        # type: (...) -> None

        medias_data = stored_hashes_json.get('images') or []
        medias_data += stored_hashes_json.get('banner', {}).get('images') or []
        medias_data += (stored_hashes_json.get('videos') or [])
        for img_hash in hashes_to_delete:
            for img_data in medias_data:
                if img_hash == img_data.get('hashImage'):
                    file_path = os.path.join(
                        self.images_path,
                        img_data.get('path').replace('/sui/images\\', ''),
                    )
                    self.updater_service.delete_file(path=file_path)

    def _update_from_remote_medias(
        self,
        totem_configs,          # type: dict
        stored_medias_hash,     # type: UpdateMediaJson
        medias_to_download,     # type: List[MediaUpdateData]
        is_banner=False,        # type: Optional[bool]
    ):
        # type: (...) -> None

        if is_banner:
            stored_hashes = stored_medias_hash.banners_totem.copy()
        else:
            stored_hashes = stored_medias_hash.images_totem.copy()
            stored_hashes.update(stored_medias_hash.videos_totem)

        medias_to_delete = self._get_medias_to_delete(
            totem_configs=totem_configs,
            stored_image_hashes=stored_hashes.values(),
            is_banner=is_banner,
        )

        if medias_to_delete:
            medias_hashes_stored_json = self._get_medias_hashes_from_stored_json()
            self._delete_medias_files(
                stored_hashes_json=medias_hashes_stored_json,
                hashes_to_delete=medias_to_delete,
            )

            medias_hashes = self._remove_and_update_config(
                totem_configs=totem_configs,
                medias_to_download=medias_to_download,
                medias_hash=medias_hashes_stored_json,
                medias_hashes_from_json=stored_hashes,
                is_banner=is_banner,
                delete_hashes=medias_to_delete,
            )

            if is_banner:
                stored_medias_hash.banners_totem = medias_hashes.get('images')
            else:
                stored_medias_hash.images_totem = medias_hashes.get('images')
                stored_medias_hash.videos_totem = medias_hashes.get('videos')

            self._save_file(update_data=stored_medias_hash)

    def _download_totem_images(self, totem_configs, stored_images_hash):
        # type: (dict, UpdateMediaJson) -> bool

        self.logger.info("Trying to update totem_stored_hashs totem media...")

        some_totem_image_downloaded = False
        totem_stored_hashes = stored_images_hash.images_totem.copy()
        totem_stored_hashes.update(stored_images_hash.videos_totem)
        medias_to_download = self._get_medias_to_download(
            totem_configs=totem_configs,
            downloaded_medias=totem_stored_hashes.values(),
        )

        self._update_from_remote_medias(
            totem_configs=totem_configs,
            stored_medias_hash=stored_images_hash,
            medias_to_download=medias_to_download,
        )

        if len(medias_to_download) > 0:
            medias_to_download = self._download_and_save(
                images_to_download=medias_to_download,
                is_totem_image=True,
            )

            hashes = self._get_medias_hashes_from_stored_json()
            medias_totem = self._remove_and_update_config(
                totem_configs=totem_configs,
                medias_to_download=medias_to_download,
                medias_hash=hashes,
                medias_hashes_from_json=totem_stored_hashes,
                is_banner=False,
            )
            stored_images_hash.images_totem = medias_totem.get('images')
            stored_images_hash.videos_totem = medias_totem.get('videos')
            self._save_file(update_data=stored_images_hash)
            some_totem_image_downloaded = True

        return some_totem_image_downloaded

    def _download_and_save(
        self,
        images_to_download,     # type: (List[MediaUpdateData]),
        is_totem_image=False    # type: bool
    ):
        # type: (...) -> List[MediaUpdateData]

        self._get_images_permissions(images_to_download)
        return self._download_and_store_images(
            images_to_download=images_to_download,
            is_totem_image=is_totem_image,
        )

    def _generate_updated_hashes_dict(
        self,
        totem_configs,          # type: dict
        medias_to_download,     # type: List[MediaUpdateData]
        delete_hashes,          # type: list
        is_banner=False,        # type: bool
    ):
        # type: (...) -> dict

        hashes_from_json = self._get_images_hashes_from_json()
        images = hashes_from_json.images_totem if not is_banner else hashes_from_json.banners_totem
        videos = hashes_from_json.videos_totem
        updated_hashes = {
            "images": {image_hash: image_hash for image_hash in images if image_hash not in delete_hashes},
            "videos": {video_hash: video_hash for video_hash in videos if video_hash not in delete_hashes},
        }

        medias_data = totem_configs if not is_banner else totem_configs.get('banner', {}) or {}
        medias_data = medias_data.get('images', []) + medias_data.get('videos', [])

        for media_data in medias_data:
            for media_to_download in medias_to_download:
                same_hash = media_data.get('hashImage') == media_to_download.image_hash
                same_name = media_data.get('name') == media_to_download.image_name
                if same_hash and same_name and media_to_download.successfully_downloaded:
                    key = 'videos' if media_data.get('name').split('.')[1] in allowed_video_extensions else 'images'
                    updated_hashes[key][media_data.get('hashImage')] = media_data.get('hashImage')

        return updated_hashes

    def _remove_and_update_config(
        self,
        totem_configs,              # type: Dict
        medias_to_download,         # type: List[MediaUpdateData]
        medias_hash,                # type: Dict
        medias_hashes_from_json,    # type: Dict
        is_banner,                  # type: bool
        delete_hashes=None,         # type: Optional[List]
    ):
        # type: (...) -> Dict

        if delete_hashes is None:
            delete_hashes = []
        old_data_hashes = deepcopy(medias_hash)
        self._update_json_totem_config(totem_configs, medias_to_download, medias_hash, is_banner)
        downloaded = [x.image_hash for x in medias_to_download if x.downloaded]
        update_images_hashes = self._update_images_hashes_json(downloaded, medias_hashes_from_json, medias_to_download)

        if is_banner:
            self._remove_old_banners_from_path(totem_configs, old_data_hashes, update_images_hashes)

        return self._generate_updated_hashes_dict(
            totem_configs=totem_configs,
            medias_to_download=medias_to_download,
            delete_hashes=delete_hashes,
            is_banner=is_banner,
        )

    def _get_medias_to_download(
        self,                   # type: MediaUpdaterService
        totem_configs,          # type: Dict
        downloaded_medias,      # type: Any[Union[List, Dict]]
    ):
        # type: (...) -> List[MediaUpdateData]

        if totem_configs is None:
            return []

        medias = totem_configs.get("images", []) + totem_configs.get("videos", [])
        payment_images = totem_configs.get("payments", {})
        if payment_images is None:
            return []

        for payment_image in payment_images.values():
            medias.append(payment_image)

        new_medias_to_download = []
        if len(medias) > 0:
            new_medias_to_download = self._get_banner_media_update(medias, downloaded_medias)

        return new_medias_to_download

    @staticmethod
    def _get_banner_media_update(list_data, downloaded_images):
        # type: (dict, Any[Union[List, Dict]]) -> List

        formatted_images_to_download = []
        for data in list_data:
            if data.get("hashImage") not in downloaded_images:
                update_data = MediaUpdateData()
                update_data.product_code = data.get("productId", None)
                update_data.image_name = data.get("name", "")
                update_data.image_hash = str(data.get("hashImage", None))
                update_data.product_type = "TT"
                formatted_images_to_download.append(update_data)

        return formatted_images_to_download

    def _remove_old_banners_from_path(self, images_totem, json_images_client, update_images_hashes):
        # type: (Dict, Dict, Dict) -> None

        images_client = self._get_medias_from_json(
            json_medias_client=json_images_client,
            key="images",
            is_banner=True
        )
        images_server = self._get_medias_from_json(
            json_medias_client=images_totem,
            key="images",
            is_banner=True
        )
        for hash_image in images_client.keys():
            if hash_image not in images_server.keys():
                image = images_client[hash_image]
                image_name = image.get("name", "")
                image_path = self._get_image_path(image_name)
                if os.path.isfile(image_path):
                    os.remove(image_path)

                continue

            if hash_image not in update_images_hashes.keys():
                update_images_hashes[hash_image] = hash_image

    def _update_images_hashes_json(self, images_hashes_from_database, images_hashes_from_json, images_to_update):
        # type: (Dict, Dict, List[MediaUpdateData]) -> Dict

        self.logger.info("Updating [imagesHashes.json]...")

        for image in images_to_update:
            if image.downloaded:
                key_code = image.product_code if image.product_type != "TT" else image.image_hash
                images_hashes_from_json[key_code] = image.image_hash

        for key_code in images_hashes_from_json.keys():
            if key_code not in images_hashes_from_database:
                del images_hashes_from_json[key_code]

        return self._delete_images_hashes_json(images_hashes_from_database, images_hashes_from_json)

    @staticmethod
    def _delete_images_hashes_json(images_hashes_from_database, images_hashes_from_json):
        # type: (Dict, Dict) -> Dict

        for key_code in images_hashes_from_json.keys():
            if key_code not in images_hashes_from_database:
                del images_hashes_from_json[key_code]

        return images_hashes_from_json

    def _update_json_totem_config(self, medias_from_server, downloaded_medias, json_medias_client, is_banner=True):
        # type: (dict, List[MediaUpdateData], dict, bool) -> None

        downloaded = [x for x in downloaded_medias if x.downloaded]
        images = self._get_updated_client_media(
            key="images",
            downloaded=downloaded,
            json_medias_client=json_medias_client,
            medias_from_server=medias_from_server,
            is_banner=is_banner,
        )

        videos = self._get_updated_client_media(
            key="videos",
            downloaded=downloaded,
            json_medias_client=json_medias_client,
            medias_from_server=medias_from_server,
            is_banner=is_banner,
        )

        if is_banner:
            if not json_medias_client.get("banner"):
                json_medias_client["banner"] = {}

            json_medias_client["banner"]["images"] = images
        else:
            json_medias_client["images"] = images
            json_medias_client["videos"] = videos

        change_time = 30
        if medias_from_server is not None:
            banner = medias_from_server.get("banner", None)
            change_time = banner.get("changeTime", 30) if banner is not None else change_time

        json_medias_client["changeTime"] = change_time
        with open(self.control_json_totem_path, "w") as json_file:
            json.dump(json_medias_client, json_file)

    def _get_updated_client_media(
        self,
        key,                   # type: str
        downloaded,            # type: List[MediaUpdateData]
        json_medias_client,    # type: dict
        medias_from_server,    # type: dict
        is_banner,             # type: bool

    ):
        # type: (...) -> List[Dict]

        media_client = self._get_medias_from_json(
            json_medias_client=json_medias_client,
            key=key,
            is_banner=is_banner,
        )
        media_server = self._get_medias_from_json(
            json_medias_client=medias_from_server,
            key=key,
            is_banner=is_banner,
        )

        medias = []
        for hash_media in media_server.keys():
            if hash_media in media_client.keys():
                medias.append(media_client[hash_media])
                continue
            data = self._get_downloaded_medias(
                hash_media=hash_media,
                downloaded=downloaded,
            )
            if data is not None:
                medias.append(data)

        return medias

    def _get_downloaded_medias(self, hash_media, downloaded):
        # type: (str, List[MediaUpdateData]) -> Dict

        for image in downloaded:
            if image.image_hash == hash_media:
                name = image.image_name
                image_path = self.images_path[self.images_path.index("/sui"):]
                path = os.path.join(image_path, image.image_name)
                return {
                    'name': name,
                    'hashImage': image.image_hash,
                    'path': path,
                    'productId': image.product_code
                }

    @staticmethod
    def _get_medias_from_json(
        json_medias_client,     # type: Dict
        key,                    # type: str
        is_banner,              # type: bool
    ):
        # type: (...) -> dict

        medias_client = json_medias_client
        if is_banner:
            medias_client = medias_client.get("banner", {})
        medias = medias_client.get(key, []) if medias_client is not None else []

        medias_downloaded = dict()
        for media in medias:
            medias_downloaded[str(media.get('hashImage'))] = media

        return medias_downloaded

    def _format_images_to_update_data(self, images_to_download, database_navigation_hash):
        # type: (dict, dict) -> List[MediaUpdateData]

        formatted_images_to_download = list()
        for product_code in images_to_download:
            update_data = MediaUpdateData()
            update_data.product_code = product_code
            update_data.image_hash = images_to_download[product_code]
            update_data.product_type = "NAV" if product_code in database_navigation_hash else "PRT"

            formatted_images_to_download.append(update_data)

        products_to_download = [x.product_code for x in formatted_images_to_download]
        self.logger.info("Trying to obtain images for products: {}".format(products_to_download))

        return formatted_images_to_download

    def _get_images_hashes_from_json(self):
        # type: () -> UpdateMediaJson

        images_hashes_from_json = None
        try:
            images_hashes_from_json = UpdateMediaJson()
            if os.path.isfile(self.control_json_path):
                with open(self.control_json_path) as json_file:
                    json_load = json.load(json_file)
                    if json_load is not None:
                        images_hashes_from_json.catalog = json_load["catalog"]
                        images_hashes_from_json.navigation = json_load["navigation"]
                        images_hashes_from_json.banners_totem = json_load["banners_totem"]
                        images_hashes_from_json.images_totem = json_load["images_totem"]
                        images_hashes_from_json.videos_totem = json_load.get("videos_totem", {})

            return images_hashes_from_json
        except Exception as _:
            os.remove(self.control_json_path)
            return images_hashes_from_json

    def _get_medias_hashes_from_stored_json(self):
        # type: () -> Dict

        try:
            if os.path.isfile(self.control_json_totem_path):
                with open(self.control_json_totem_path) as json_file:
                    return json.load(json_file)

            return {}
        except Exception as _:
            return {}

    @staticmethod
    def _filter_images_to_download(images_hashes_from_database, images_hashes_from_json):
        # type: (Dict, Dict) -> Dict

        product_that_we_need_to_download = {}
        images = images_hashes_from_json
        for product_code in images_hashes_from_database:
            database_hash = images_hashes_from_database[product_code]
            if images is None or product_code not in images or database_hash != images[product_code]:
                product_that_we_need_to_download[product_code] = database_hash

        return product_that_we_need_to_download

    def _get_images_permissions(self, images_to_download):
        # type: (List[MediaUpdateData]) -> None

        try:
            hashes_to_get_permission = []
            for image_to_download in images_to_download:
                hashes_to_get_permission.append(image_to_download.image_hash)

            images_permissions = self.api_repository.get_images_permissions(hashes_to_get_permission)

            for image_permission in images_permissions:
                obtained_image_hash = image_permission.get("imageHash")
                for image_to_download in images_to_download:
                    if image_to_download.image_hash == obtained_image_hash:
                        image_to_download.link_to_download = image_permission.get("link")
                        image_to_download.expiration_date = iso8601.parse_date(image_permission.get("expirationDate"))

        except Exception as ex:
            raise ErrorRetrievingPendingUpdate(ex)

    def _get_totem_configs(self):
        # type: () -> Dict

        try:
            return self.api_repository.get_totem_config()
        except Exception as ex:
            raise ErrorGettingTotemConfig(ex)

    def _download_and_store_images(
        self,
        images_to_download,     # type: List[MediaUpdateData]
        is_totem_image=False    # type: bool
    ):
        # type: (...) -> List[MediaUpdateData]

        backup_file_name = None
        for image_to_download in images_to_download:
            try:
                now = self._get_utc_now()
                image_type = " product" if image_to_download.product_code else ""
                name = image_to_download.image_name
                if image_to_download.product_code:
                    name = image_to_download.product_code

                if image_to_download.expiration_date and now > image_to_download.expiration_date:
                    self.logger.error("Expiration date reached for {}: {}".format(image_type, name))
                    continue

                if not image_to_download.link_to_download:
                    self.logger.error("Image has no link to download: {}".format(name))
                    continue

                package = self.api_repository.download_image(image_to_download.link_to_download)
                image_to_download.successfully_downloaded = package.ok
                if not package.ok:
                    self.logger.error("Could not download image for {}: {}".format(image_type, name))
                    continue

                image = image_to_download
                if image.product_type == "TT":
                    product_image_name = image.image_name
                else:
                    product_image_name = self._format_product_image_name(image.product_code, image.product_type)
                product_image_path = self._get_image_path(product_image_name)

                if is_totem_image:
                    backup_file_name = self._backup_file(product_image_path)

                self._store_image(product_image_path, package)
                self._remove_backup_file(backup_file_name)

                image_to_download.downloaded = True

                self.logger.info("Image downloaded and stored for product: {}".format(image_to_download.product_code))
            except Exception as _:
                self._restore_backup_file(backup_file_name)
                self.logger.info("Image downloaded and stored for product: {}".format(image_to_download.product_code))
            finally:
                backup_file_name = None

        return images_to_download

    def _get_image_path(self, prt_image_name):
        # type: (str) -> str

        return os.path.join(self.images_path, prt_image_name)

    @staticmethod
    def _format_product_image_name(product_code, product_type):
        # type: (str, bool) -> str

        return "{}{}.png".format(product_type, product_code.zfill(8))

    @staticmethod
    def _store_image(image_path, package):
        # type: (str, Response) -> None

        with open(image_path, "wb") as image_file:
            for chunk in package.iter_content(chunk_size=128):
                image_file.write(chunk)

    def _save_file(self, update_data):
        with open(self.control_json_path, "w") as json_file:
            json.dump(vars(update_data), json_file)

    @staticmethod
    def _backup_file(
            image_path  # type: str
    ):
        # type: (...) -> str

        image_part = image_path.split('.')
        image_part = "{}_old.{}".format(image_part[0], image_part[1])
        if os.path.exists(image_path):
            os.rename(image_path, image_part)

        return image_part

    @staticmethod
    def _restore_backup_file(
            image_path  # type: str
    ):
        # type: (...) -> None

        if image_path:
            os.rename(image_path, image_path.replace("_old", ""))

    @staticmethod
    def _remove_backup_file(
            image_path  # type: str
    ):
        # type: (...) -> None

        if image_path is not None and os.path.isfile(image_path):
            os.remove(image_path)

    @staticmethod
    def _get_utc_now():
        # type: () -> datetime

        return pytz.UTC.localize(datetime.utcnow())



from typing import Optional

from application.service import MediaUpdaterService


class MediaUpdaterInteractor(object):
    def __init__(
        self,
        media_updater_service,  # type: MediaUpdaterService
    ):
        # type: (...) -> None

        self.media_updater_service = media_updater_service

    def update_media(
        self,
        force_update=False  # type: Optional[bool]
    ):
        # type: (bool) -> None

        return self.media_updater_service.update_media(force_update=force_update)



# -*- coding: utf-8 -*-
import os
import component
from helper import (
    config_logger,
    import_pydevd,
)


def main():
    import_pydevd(os.environ["LOADERCFG"], 9134)
    config_logger(os.environ["LOADERCFG"], "TableManager", max_files=25)

    component.main()



# Copyright 2016 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import copy
import logging
import re
import threading

from botocore.compat import six

from s3transfer.constants import KB, MB
from s3transfer.constants import ALLOWED_DOWNLOAD_ARGS
from s3transfer.utils import get_callbacks
from s3transfer.utils import signal_transferring
from s3transfer.utils import signal_not_transferring
from s3transfer.utils import CallArgs
from s3transfer.utils import OSUtils
from s3transfer.utils import TaskSemaphore
from s3transfer.utils import SlidingWindowSemaphore
from s3transfer.exceptions import CancelledError
from s3transfer.exceptions import FatalError
from s3transfer.futures import IN_MEMORY_DOWNLOAD_TAG
from s3transfer.futures import IN_MEMORY_UPLOAD_TAG
from s3transfer.futures import BoundedExecutor
from s3transfer.futures import TransferFuture
from s3transfer.futures import TransferMeta
from s3transfer.futures import TransferCoordinator
from s3transfer.download import DownloadSubmissionTask
from s3transfer.upload import UploadSubmissionTask
from s3transfer.copies import CopySubmissionTask
from s3transfer.delete import DeleteSubmissionTask
from s3transfer.bandwidth import LeakyBucket
from s3transfer.bandwidth import BandwidthLimiter


logger = logging.getLogger(__name__)


class TransferConfig(object):
    def __init__(self,
                 multipart_threshold=8 * MB,
                 multipart_chunksize=8 * MB,
                 max_request_concurrency=10,
                 max_submission_concurrency=5,
                 max_request_queue_size=1000,
                 max_submission_queue_size=1000,
                 max_io_queue_size=1000,
                 io_chunksize=256 * KB,
                 num_download_attempts=5,
                 max_in_memory_upload_chunks=10,
                 max_in_memory_download_chunks=10,
                 max_bandwidth=None):
        """Configurations for the transfer mangager

        :param multipart_threshold: The threshold for which multipart
            transfers occur.

        :param max_request_concurrency: The maximum number of S3 API
            transfer-related requests that can happen at a time.

        :param max_submission_concurrency: The maximum number of threads
            processing a call to a TransferManager method. Processing a
            call usually entails determining which S3 API requests that need
            to be enqueued, but does **not** entail making any of the
            S3 API data transfering requests needed to perform the transfer.
            The threads controlled by ``max_request_concurrency`` is
            responsible for that.

        :param multipart_chunksize: The size of each transfer if a request
            becomes a multipart transfer.

        :param max_request_queue_size: The maximum amount of S3 API requests
            that can be queued at a time. A value of zero means that there
            is no maximum.

        :param max_submission_queue_size: The maximum amount of
            TransferManager method calls that can be queued at a time. A value
            of zero means that there is no maximum.

        :param max_io_queue_size: The maximum amount of read parts that
            can be queued to be written to disk per download. A value of zero
            means that there is no maximum. The default size for each element
            in this queue is 8 KB.

        :param io_chunksize: The max size of each chunk in the io queue.
            Currently, this is size used when reading from the downloaded
            stream as well.

        :param num_download_attempts: The number of download attempts that
            will be tried upon errors with downloading an object in S3. Note
            that these retries account for errors that occur when streamming
            down the data from s3 (i.e. socket errors and read timeouts that
            occur after recieving an OK response from s3).
            Other retryable exceptions such as throttling errors and 5xx errors
            are already retried by botocore (this default is 5). The
            ``num_download_attempts`` does not take into account the
            number of exceptions retried by botocore.

        :param max_in_memory_upload_chunks: The number of chunks that can
            be stored in memory at a time for all ongoing upload requests.
            This pertains to chunks of data that need to be stored in memory
            during an upload if the data is sourced from a file-like object.
            The total maximum memory footprint due to a in-memory upload
            chunks is roughly equal to:

                max_in_memory_upload_chunks * multipart_chunksize
                + max_submission_concurrency * multipart_chunksize

            ``max_submission_concurrency`` has an affect on this value because
            for each thread pulling data off of a file-like object, they may
            be waiting with a single read chunk to be submitted for upload
            because the ``max_in_memory_upload_chunks`` value has been reached
            by the threads making the upload request.

        :param max_in_memory_download_chunks: The number of chunks that can
            be buffered in memory and **not** in the io queue at a time for all
            ongoing dowload requests. This pertains specifically to file-like
            objects that cannot be seeked. The total maximum memory footprint
            due to a in-memory download chunks is roughly equal to:

                max_in_memory_download_chunks * multipart_chunksize

        :param max_bandwidth: The maximum bandwidth that will be consumed
            in uploading and downloading file content. The value is in terms of
            bytes per second.
        """
        self.multipart_threshold = multipart_threshold
        self.multipart_chunksize = multipart_chunksize
        self.max_request_concurrency = max_request_concurrency
        self.max_submission_concurrency = max_submission_concurrency
        self.max_request_queue_size = max_request_queue_size
        self.max_submission_queue_size = max_submission_queue_size
        self.max_io_queue_size = max_io_queue_size
        self.io_chunksize = io_chunksize
        self.num_download_attempts = num_download_attempts
        self.max_in_memory_upload_chunks = max_in_memory_upload_chunks
        self.max_in_memory_download_chunks = max_in_memory_download_chunks
        self.max_bandwidth = max_bandwidth
        self._validate_attrs_are_nonzero()

    def _validate_attrs_are_nonzero(self):
        for attr, attr_val, in self.__dict__.items():
            if attr_val is not None and attr_val <= 0:
                raise ValueError(
                    'Provided parameter %s of value %s must be greater than '
                    '0.' % (attr, attr_val))


class TransferManager(object):
    ALLOWED_DOWNLOAD_ARGS = ALLOWED_DOWNLOAD_ARGS

    ALLOWED_UPLOAD_ARGS = [
        'ACL',
        'CacheControl',
        'ContentDisposition',
        'ContentEncoding',
        'ContentLanguage',
        'ContentType',
        'ExpectedBucketOwner',
        'Expires',
        'GrantFullControl',
        'GrantRead',
        'GrantReadACP',
        'GrantWriteACP',
        'Metadata',
        'RequestPayer',
        'ServerSideEncryption',
        'StorageClass',
        'SSECustomerAlgorithm',
        'SSECustomerKey',
        'SSECustomerKeyMD5',
        'SSEKMSKeyId',
        'SSEKMSEncryptionContext',
        'Tagging',
        'WebsiteRedirectLocation'
    ]

    ALLOWED_COPY_ARGS = ALLOWED_UPLOAD_ARGS + [
        'CopySourceIfMatch',
        'CopySourceIfModifiedSince',
        'CopySourceIfNoneMatch',
        'CopySourceIfUnmodifiedSince',
        'CopySourceSSECustomerAlgorithm',
        'CopySourceSSECustomerKey',
        'CopySourceSSECustomerKeyMD5',
        'MetadataDirective',
        'TaggingDirective',
    ]

    ALLOWED_DELETE_ARGS = [
        'MFA',
        'VersionId',
        'RequestPayer',
        'ExpectedBucketOwner'
    ]

    VALIDATE_SUPPORTED_BUCKET_VALUES = True

    _UNSUPPORTED_BUCKET_PATTERNS = {
        'S3 Object Lambda': re.compile(
            r'^arn:(aws).*:s3-object-lambda:[a-z\-0-9]+:[0-9]{12}:'
            r'accesspoint[/:][a-zA-Z0-9\-]{1,63}'
        ),
    }

    def __init__(self, client, config=None, osutil=None, executor_cls=None):
        """A transfer manager interface for Amazon S3

        :param client: Client to be used by the manager
        :param config: TransferConfig to associate specific configurations
        :param osutil: OSUtils object to use for os-related behavior when
            using with transfer manager.

        :type executor_cls: s3transfer.futures.BaseExecutor
        :param executor_cls: The class of executor to use with the transfer
            manager. By default, concurrent.futures.ThreadPoolExecutor is used.
        """
        self._client = client
        self._config = config
        if config is None:
            self._config = TransferConfig()
        self._osutil = osutil
        if osutil is None:
            self._osutil = OSUtils()
        self._coordinator_controller = TransferCoordinatorController()
        # A counter to create unique id's for each transfer submitted.
        self._id_counter = 0

        # The executor responsible for making S3 API transfer requests
        self._request_executor = BoundedExecutor(
            max_size=self._config.max_request_queue_size,
            max_num_threads=self._config.max_request_concurrency,
            tag_semaphores={
                IN_MEMORY_UPLOAD_TAG: TaskSemaphore(
                    self._config.max_in_memory_upload_chunks),
                IN_MEMORY_DOWNLOAD_TAG: SlidingWindowSemaphore(
                    self._config.max_in_memory_download_chunks)
            },
            executor_cls=executor_cls
        )

        # The executor responsible for submitting the necessary tasks to
        # perform the desired transfer
        self._submission_executor = BoundedExecutor(
            max_size=self._config.max_submission_queue_size,
            max_num_threads=self._config.max_submission_concurrency,
            executor_cls=executor_cls

        )

        # There is one thread available for writing to disk. It will handle
        # downloads for all files.
        self._io_executor = BoundedExecutor(
            max_size=self._config.max_io_queue_size,
            max_num_threads=1,
            executor_cls=executor_cls
        )

        # The component responsible for limiting bandwidth usage if it
        # is configured.
        self._bandwidth_limiter = None
        if self._config.max_bandwidth is not None:
            logger.debug(
                'Setting max_bandwidth to %s', self._config.max_bandwidth)
            leaky_bucket = LeakyBucket(self._config.max_bandwidth)
            self._bandwidth_limiter = BandwidthLimiter(leaky_bucket)

        self._register_handlers()

    @property
    def client(self):
        return self._client

    @property
    def config(self):
        return self._config

    def upload(self, fileobj, bucket, key, extra_args=None, subscribers=None):
        """Uploads a file to S3

        :type fileobj: str or seekable file-like object
        :param fileobj: The name of a file to upload or a seekable file-like
            object to upload. It is recommended to use a filename because
            file-like objects may result in higher memory usage.

        :type bucket: str
        :param bucket: The name of the bucket to upload to

        :type key: str
        :param key: The name of the key to upload to

        :type extra_args: dict
        :param extra_args: Extra arguments that may be passed to the
            client operation

        :type subscribers: list(s3transfer.subscribers.BaseSubscriber)
        :param subscribers: The list of subscribers to be invoked in the
            order provided based on the event emit during the process of
            the transfer request.

        :rtype: s3transfer.futures.TransferFuture
        :returns: Transfer future representing the upload
        """
        if extra_args is None:
            extra_args = {}
        if subscribers is None:
            subscribers = []
        self._validate_all_known_args(extra_args, self.ALLOWED_UPLOAD_ARGS)
        self._validate_if_bucket_supported(bucket)
        call_args = CallArgs(
            fileobj=fileobj, bucket=bucket, key=key, extra_args=extra_args,
            subscribers=subscribers
        )
        extra_main_kwargs = {}
        if self._bandwidth_limiter:
            extra_main_kwargs['bandwidth_limiter'] = self._bandwidth_limiter
        return self._submit_transfer(
            call_args, UploadSubmissionTask, extra_main_kwargs)

    def download(self, bucket, key, fileobj, extra_args=None,
                 subscribers=None):
        """Downloads a file from S3

        :type bucket: str
        :param bucket: The name of the bucket to download from

        :type key: str
        :param key: The name of the key to download from

        :type fileobj: str or seekable file-like object
        :param fileobj: The name of a file to download or a seekable file-like
            object to download. It is recommended to use a filename because
            file-like objects may result in higher memory usage.

        :type extra_args: dict
        :param extra_args: Extra arguments that may be passed to the
            client operation

        :type subscribers: list(s3transfer.subscribers.BaseSubscriber)
        :param subscribers: The list of subscribers to be invoked in the
            order provided based on the event emit during the process of
            the transfer request.

        :rtype: s3transfer.futures.TransferFuture
        :returns: Transfer future representing the download
        """
        if extra_args is None:
            extra_args = {}
        if subscribers is None:
            subscribers = []
        self._validate_all_known_args(extra_args, self.ALLOWED_DOWNLOAD_ARGS)
        self._validate_if_bucket_supported(bucket)
        call_args = CallArgs(
            bucket=bucket, key=key, fileobj=fileobj, extra_args=extra_args,
            subscribers=subscribers
        )
        extra_main_kwargs = {'io_executor': self._io_executor}
        if self._bandwidth_limiter:
            extra_main_kwargs['bandwidth_limiter'] = self._bandwidth_limiter
        return self._submit_transfer(
            call_args, DownloadSubmissionTask, extra_main_kwargs)

    def copy(self, copy_source, bucket, key, extra_args=None,
             subscribers=None, source_client=None):
        """Copies a file in S3

        :type copy_source: dict
        :param copy_source: The name of the source bucket, key name of the
            source object, and optional version ID of the source object. The
            dictionary format is:
            ``{'Bucket': 'bucket', 'Key': 'key', 'VersionId': 'id'}``. Note
            that the ``VersionId`` key is optional and may be omitted.

        :type bucket: str
        :param bucket: The name of the bucket to copy to

        :type key: str
        :param key: The name of the key to copy to

        :type extra_args: dict
        :param extra_args: Extra arguments that may be passed to the
            client operation

        :type subscribers: a list of subscribers
        :param subscribers: The list of subscribers to be invoked in the
            order provided based on the event emit during the process of
            the transfer request.

        :type source_client: botocore or boto3 Client
        :param source_client: The client to be used for operation that
            may happen at the source object. For example, this client is
            used for the head_object that determines the size of the copy.
            If no client is provided, the transfer manager's client is used
            as the client for the source object.

        :rtype: s3transfer.futures.TransferFuture
        :returns: Transfer future representing the copy
        """
        if extra_args is None:
            extra_args = {}
        if subscribers is None:
            subscribers = []
        if source_client is None:
            source_client = self._client
        self._validate_all_known_args(extra_args, self.ALLOWED_COPY_ARGS)
        if isinstance(copy_source, dict):
            self._validate_if_bucket_supported(copy_source.get('Bucket'))
        self._validate_if_bucket_supported(bucket)
        call_args = CallArgs(
            copy_source=copy_source, bucket=bucket, key=key,
            extra_args=extra_args, subscribers=subscribers,
            source_client=source_client
        )
        return self._submit_transfer(call_args, CopySubmissionTask)

    def delete(self, bucket, key, extra_args=None, subscribers=None):
        """Delete an S3 object.

        :type bucket: str
        :param bucket: The name of the bucket.

        :type key: str
        :param key: The name of the S3 object to delete.

        :type extra_args: dict
        :param extra_args: Extra arguments that may be passed to the
            DeleteObject call.

        :type subscribers: list
        :param subscribers: A list of subscribers to be invoked during the
            process of the transfer request.  Note that the ``on_progress``
            callback is not invoked during object deletion.

        :rtype: s3transfer.futures.TransferFuture
        :return: Transfer future representing the deletion.

        """
        if extra_args is None:
            extra_args = {}
        if subscribers is None:
            subscribers = []
        self._validate_all_known_args(extra_args, self.ALLOWED_DELETE_ARGS)
        self._validate_if_bucket_supported(bucket)
        call_args = CallArgs(
            bucket=bucket, key=key, extra_args=extra_args,
            subscribers=subscribers
        )
        return self._submit_transfer(call_args, DeleteSubmissionTask)

    def _validate_if_bucket_supported(self, bucket):
        # s3 high level operations don't support some resources
        # (eg. S3 Object Lambda) only direct API calls are available
        # for such resources
        if self.VALIDATE_SUPPORTED_BUCKET_VALUES:
            for resource, pattern in self._UNSUPPORTED_BUCKET_PATTERNS.items():
                match = pattern.match(bucket)
                if match:
                    raise ValueError(
                        'TransferManager methods do not support %s '
                        'resource. Use direct client calls instead.' % resource
                    )

    def _validate_all_known_args(self, actual, allowed):
        for kwarg in actual:
            if kwarg not in allowed:
                raise ValueError(
                    "Invalid extra_args key '%s', "
                    "must be one of: %s" % (
                        kwarg, ', '.join(allowed)))

    def _submit_transfer(self, call_args, submission_task_cls,
                         extra_main_kwargs=None):
        if not extra_main_kwargs:
            extra_main_kwargs = {}

        # Create a TransferFuture to return back to the user
        transfer_future, components = self._get_future_with_components(
            call_args)

        # Add any provided done callbacks to the created transfer future
        # to be invoked on the transfer future being complete.
        for callback in get_callbacks(transfer_future, 'done'):
            components['coordinator'].add_done_callback(callback)

        # Get the main kwargs needed to instantiate the submission task
        main_kwargs = self._get_submission_task_main_kwargs(
            transfer_future, extra_main_kwargs)

        # Submit a SubmissionTask that will submit all of the necessary
        # tasks needed to complete the S3 transfer.
        self._submission_executor.submit(
            submission_task_cls(
                transfer_coordinator=components['coordinator'],
                main_kwargs=main_kwargs
            )
        )

        # Increment the unique id counter for future transfer requests
        self._id_counter += 1

        return transfer_future

    def _get_future_with_components(self, call_args):
        transfer_id = self._id_counter
        # Creates a new transfer future along with its components
        transfer_coordinator = TransferCoordinator(transfer_id=transfer_id)
        # Track the transfer coordinator for transfers to manage.
        self._coordinator_controller.add_transfer_coordinator(
            transfer_coordinator)
        # Also make sure that the transfer coordinator is removed once
        # the transfer completes so it does not stick around in memory.
        transfer_coordinator.add_done_callback(
            self._coordinator_controller.remove_transfer_coordinator,
            transfer_coordinator)
        components = {
            'meta': TransferMeta(call_args, transfer_id=transfer_id),
            'coordinator': transfer_coordinator
        }
        transfer_future = TransferFuture(**components)
        return transfer_future, components

    def _get_submission_task_main_kwargs(
            self, transfer_future, extra_main_kwargs):
        main_kwargs = {
            'client': self._client,
            'config': self._config,
            'osutil': self._osutil,
            'request_executor': self._request_executor,
            'transfer_future': transfer_future
        }
        main_kwargs.update(extra_main_kwargs)
        return main_kwargs

    def _register_handlers(self):
        # Register handlers to enable/disable callbacks on uploads.
        event_name = 'request-created.s3'
        self._client.meta.events.register_first(
            event_name, signal_not_transferring,
            unique_id='s3upload-not-transferring')
        self._client.meta.events.register_last(
            event_name, signal_transferring,
            unique_id='s3upload-transferring')

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, *args):
        cancel = False
        cancel_msg = ''
        cancel_exc_type = FatalError
        # If a exception was raised in the context handler, signal to cancel
        # all of the inprogress futures in the shutdown.
        if exc_type:
            cancel = True
            cancel_msg = six.text_type(exc_value)
            if not cancel_msg:
                cancel_msg = repr(exc_value)
            # If it was a KeyboardInterrupt, the cancellation was initiated
            # by the user.
            if isinstance(exc_value, KeyboardInterrupt):
                cancel_exc_type = CancelledError
        self._shutdown(cancel, cancel_msg, cancel_exc_type)

    def shutdown(self, cancel=False, cancel_msg=''):
        """Shutdown the TransferManager

        It will wait till all transfers complete before it completely shuts
        down.

        :type cancel: boolean
        :param cancel: If True, calls TransferFuture.cancel() for
            all in-progress in transfers. This is useful if you want the
            shutdown to happen quicker.

        :type cancel_msg: str
        :param cancel_msg: The message to specify if canceling all in-progress
            transfers.
        """
        self._shutdown(cancel, cancel, cancel_msg)

    def _shutdown(self, cancel, cancel_msg, exc_type=CancelledError):
        if cancel:
            # Cancel all in-flight transfers if requested, before waiting
            # for them to complete.
            self._coordinator_controller.cancel(cancel_msg, exc_type)
        try:
            # Wait until there are no more in-progress transfers. This is
            # wrapped in a try statement because this can be interrupted
            # with a KeyboardInterrupt that needs to be caught.
            self._coordinator_controller.wait()
        except KeyboardInterrupt:
            # If not errors were raised in the try block, the cancel should
            # have no coordinators it needs to run cancel on. If there was
            # an error raised in the try statement we want to cancel all of
            # the inflight transfers before shutting down to speed that
            # process up.
            self._coordinator_controller.cancel('KeyboardInterrupt()')
            raise
        finally:
            # Shutdown all of the executors.
            self._submission_executor.shutdown()
            self._request_executor.shutdown()
            self._io_executor.shutdown()


class TransferCoordinatorController(object):
    def __init__(self):
        """Abstraction to control all transfer coordinators

        This abstraction allows the manager to wait for inprogress transfers
        to complete and cancel all inprogress transfers.
        """
        self._lock = threading.Lock()
        self._tracked_transfer_coordinators = set()

    @property
    def tracked_transfer_coordinators(self):
        """The set of transfer coordinators being tracked"""
        with self._lock:
            # We return a copy because the set is mutable and if you were to
            # iterate over the set, it may be changing in length due to
            # additions and removals of transfer coordinators.
            return copy.copy(self._tracked_transfer_coordinators)

    def add_transfer_coordinator(self, transfer_coordinator):
        """Adds a transfer coordinator of a transfer to be canceled if needed

        :type transfer_coordinator: s3transfer.futures.TransferCoordinator
        :param transfer_coordinator: The transfer coordinator for the
            particular transfer
        """
        with self._lock:
            self._tracked_transfer_coordinators.add(transfer_coordinator)

    def remove_transfer_coordinator(self, transfer_coordinator):
        """Remove a transfer coordinator from cancelation consideration

        Typically, this method is invoked by the transfer coordinator itself
        to remove its self when it completes its transfer.

        :type transfer_coordinator: s3transfer.futures.TransferCoordinator
        :param transfer_coordinator: The transfer coordinator for the
            particular transfer
        """
        with self._lock:
            self._tracked_transfer_coordinators.remove(transfer_coordinator)

    def cancel(self, msg='', exc_type=CancelledError):
        """Cancels all inprogress transfers

        This cancels the inprogress transfers by calling cancel() on all
        tracked transfer coordinators.

        :param msg: The message to pass on to each transfer coordinator that
            gets cancelled.

        :param exc_type: The type of exception to set for the cancellation
        """
        for transfer_coordinator in self.tracked_transfer_coordinators:
            transfer_coordinator.cancel(msg, exc_type)

    def wait(self):
        """Wait until there are no more inprogress transfers

        This will not stop when failures are encountered and not propogate any
        of these errors from failed transfers, but it can be interrupted with
        a KeyboardInterrupt.
        """
        try:
            transfer_coordinator = None
            for transfer_coordinator in self.tracked_transfer_coordinators:
                transfer_coordinator.result()
        except KeyboardInterrupt:
            logger.debug('Received KeyboardInterrupt in wait()')
            # If Keyboard interrupt is raised while waiting for
            # the result, then exit out of the wait and raise the
            # exception
            if transfer_coordinator:
                logger.debug(
                    'On KeyboardInterrupt was waiting for %s',
                    transfer_coordinator)
            raise
        except Exception:
            # A general exception could have been thrown because
            # of result(). We just want to ignore this and continue
            # because we at least know that the transfer coordinator
            # has completed.
            pass



# -*- coding: utf-8 -*-
import os

from helper import config_logger, import_pydevd

from messagehandler import MessageHandler
from msgbus import MBEasyContext
from pos_model import OrderParser
from eventhandler import NTaxCalcEventHandler
from taxcalculator import TaxCalculatorService, GeneralTaxCalculator, OrderFormatter, TaxItemFormatter

REQUIRED_SERVICES = ""


def main():
    loader_path = os.environ["LOADERCFG"]
    import_pydevd(loader_path, 9175)

    config_logger(loader_path, 'NTaxCalc')
    mbcontext = MBEasyContext("NTaxCalc")

    message_handler = MessageHandler(mbcontext, "NTaxCalc", "NTaxCalc", REQUIRED_SERVICES)

    order_parser = OrderParser()

    general_tax_calculator = GeneralTaxCalculator(mbcontext)

    tax_item_formatter = TaxItemFormatter()
    order_formatter = OrderFormatter(tax_item_formatter)
    tax_calculator_service = TaxCalculatorService(order_parser, general_tax_calculator, order_formatter, mbcontext)

    event_handler = NTaxCalcEventHandler(mbcontext, tax_calculator_service)

    message_handler.set_event_handler(event_handler)

    message_handler.subscribe_sync_events(["TAXCALC", "TAX_CALC", "TAX_CALC_ITEM"])

    message_handler.handle_events()



from ._CatalogUpdaterManager import CatalogUpdaterManager
from ._UserUpdaterManager import UserUpdaterManager
from ._LoaderUpdaterManager import LoaderUpdaterManager
from ._BinaryUpdaterManager import BinaryUpdaterManager
from ._MediaUpdaterManager import MediaUpdaterManager



from typing import Dict, Any
from pos_model import Order

from old_helper import validar_cnpj, validar_cpf

from _NfceXmlPartBuilder import NfceXmlPartBuilder
from _ContextKeys import ContextKeys


class DestinatarioBuilder(NfceXmlPartBuilder):

    def __init__(self, ambiente):
        # type: (unicode) -> None

        self.ambiente = ambiente

    def build_xml(self, order, context):
        # type: (Order, Dict[unicode, Any]) -> unicode

        # Dados Destinatario (cliente)
        xml = ""
        cpf_cnpj = order.customer_cpf
        context[ContextKeys.cpf_cnpj] = cpf_cnpj

        if cpf_cnpj:
            if len(cpf_cnpj) == 11 and validar_cpf(cpf_cnpj):
                xml += "<dest><CPF>{}</CPF>".format(cpf_cnpj)
            elif len(cpf_cnpj) == 14 and validar_cnpj(cpf_cnpj):
                xml += "<dest><CNPJ>{}</CNPJ>".format(cpf_cnpj)

            if not xml:
                return ""

            if self.ambiente == "2":
                xml += "<xNome>NF-E EMITIDA EM AMBIENTE DE HOMOLOGACAO - SEM VALOR FISCAL</xNome>"

            if context[ContextKeys.delivery_partner] is not None:
                xml += self._get_address(context)

            xml += "<indIEDest>9</indIEDest>"
            xml += "</dest>"

        return xml

    @staticmethod
    def _get_address(
            context  # type: (Dict[unicode, Any])
    ):
        # type: (...) -> unicode

        delivery_partner = context[ContextKeys.delivery_partner]
        if delivery_partner is None:
            return ""

        delivery_address = delivery_partner.get("deliveryAddress")
        street_name = delivery_address.get("streetName")
        street_number = delivery_address.get("streetNumber")

        tag = "<enderDest>{0}</enderDest>"
        address = "<xLgr>{0}</xLgr><nro>{1}</nro>".format(street_name, street_number)
        complement = delivery_address.get("complement")
        if complement is not None:
            address += "<xCpl>{}</xCpl>".format(complement)

        address += "<xBairro>{}</xBairro>".format(delivery_address.get("neighborhood"))
        address += "<cMun>{}</cMun>".format(delivery_address.get("municipalityCode"))
        address += "<xMun>{}</xMun>".format(delivery_address.get("city"))
        address += "<UF>{}</UF>".format(delivery_address.get("state"))
        postal_code = delivery_address.get("postalCode", None)
        if postal_code is not None:
            address += "<CEP>{}</CEP>".format(postal_code)

        return tag.format(address)



import os

import cfgtools

pre_account_with_discount = None


def get_pre_account_with_discount():
    # type: (...) -> bool

    global pre_account_with_discount

    if pre_account_with_discount is None:
        config = cfgtools.read(os.environ["LOADERCFG"])
        pre_account_with_discount = config.find_value("Customizations.PreAccountWithDiscount", "false")

    return pre_account_with_discount.lower() == "true"



# idno.py - functions for handling Moldavian company identification numbers
# coding: utf-8
#
# Copyright (C) 2019 Arthur de Jong
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301 USA

"""IDNO (Moldavian company identification number).

The IDNO is used in Moldavia as unique identifier for legal entities. The
number consists of 13 digits. The first digit identifies the registry,
followed by three digits for the year the code was assigned. The number ends
with five identifier digits and a check digit.

More information:

* https://www.idno.md

>>> validate('1008600038413')
'1008600038413'
>>> validate('1008600038412')
Traceback (most recent call last):
    ...
InvalidChecksum: ...
"""

from stdnum.exceptions import *
from stdnum.util import clean, isdigits


def compact(number):
    """Convert the number to the minimal representation. This strips the
    number of any valid separators and removes surrounding whitespace."""
    return clean(number, ' ').strip()


def calc_check_digit(number):
    """Calculate the check digit."""
    weights = (7, 3, 1, 7, 3, 1, 7, 3, 1, 7, 3, 1)
    return str(sum(w * int(n) for w, n in zip(weights, number)) % 10)


def validate(number):
    """Check if the number provided is valid. This checks the length,
    formatting and check digit."""
    number = compact(number)
    if not isdigits(number):
        raise InvalidFormat()
    if len(number) != 13:
        raise InvalidLength()
    if number[-1] != calc_check_digit(number):
        raise InvalidChecksum()
    return number


def is_valid(number):
    """Check if the number provided is valid. This checks the length,
    formatting and check digit."""
    try:
        return bool(validate(number))
    except ValidationError:
        return False



data = (
'bols',    # 0x00
'bolt',    # 0x01
'bolp',    # 0x02
'bolh',    # 0x03
'bom',    # 0x04
'bob',    # 0x05
'bobs',    # 0x06
'bos',    # 0x07
'boss',    # 0x08
'bong',    # 0x09
'boj',    # 0x0a
'boc',    # 0x0b
'bok',    # 0x0c
'bot',    # 0x0d
'bop',    # 0x0e
'boh',    # 0x0f
'bwa',    # 0x10
'bwag',    # 0x11
'bwagg',    # 0x12
'bwags',    # 0x13
'bwan',    # 0x14
'bwanj',    # 0x15
'bwanh',    # 0x16
'bwad',    # 0x17
'bwal',    # 0x18
'bwalg',    # 0x19
'bwalm',    # 0x1a
'bwalb',    # 0x1b
'bwals',    # 0x1c
'bwalt',    # 0x1d
'bwalp',    # 0x1e
'bwalh',    # 0x1f
'bwam',    # 0x20
'bwab',    # 0x21
'bwabs',    # 0x22
'bwas',    # 0x23
'bwass',    # 0x24
'bwang',    # 0x25
'bwaj',    # 0x26
'bwac',    # 0x27
'bwak',    # 0x28
'bwat',    # 0x29
'bwap',    # 0x2a
'bwah',    # 0x2b
'bwae',    # 0x2c
'bwaeg',    # 0x2d
'bwaegg',    # 0x2e
'bwaegs',    # 0x2f
'bwaen',    # 0x30
'bwaenj',    # 0x31
'bwaenh',    # 0x32
'bwaed',    # 0x33
'bwael',    # 0x34
'bwaelg',    # 0x35
'bwaelm',    # 0x36
'bwaelb',    # 0x37
'bwaels',    # 0x38
'bwaelt',    # 0x39
'bwaelp',    # 0x3a
'bwaelh',    # 0x3b
'bwaem',    # 0x3c
'bwaeb',    # 0x3d
'bwaebs',    # 0x3e
'bwaes',    # 0x3f
'bwaess',    # 0x40
'bwaeng',    # 0x41
'bwaej',    # 0x42
'bwaec',    # 0x43
'bwaek',    # 0x44
'bwaet',    # 0x45
'bwaep',    # 0x46
'bwaeh',    # 0x47
'boe',    # 0x48
'boeg',    # 0x49
'boegg',    # 0x4a
'boegs',    # 0x4b
'boen',    # 0x4c
'boenj',    # 0x4d
'boenh',    # 0x4e
'boed',    # 0x4f
'boel',    # 0x50
'boelg',    # 0x51
'boelm',    # 0x52
'boelb',    # 0x53
'boels',    # 0x54
'boelt',    # 0x55
'boelp',    # 0x56
'boelh',    # 0x57
'boem',    # 0x58
'boeb',    # 0x59
'boebs',    # 0x5a
'boes',    # 0x5b
'boess',    # 0x5c
'boeng',    # 0x5d
'boej',    # 0x5e
'boec',    # 0x5f
'boek',    # 0x60
'boet',    # 0x61
'boep',    # 0x62
'boeh',    # 0x63
'byo',    # 0x64
'byog',    # 0x65
'byogg',    # 0x66
'byogs',    # 0x67
'byon',    # 0x68
'byonj',    # 0x69
'byonh',    # 0x6a
'byod',    # 0x6b
'byol',    # 0x6c
'byolg',    # 0x6d
'byolm',    # 0x6e
'byolb',    # 0x6f
'byols',    # 0x70
'byolt',    # 0x71
'byolp',    # 0x72
'byolh',    # 0x73
'byom',    # 0x74
'byob',    # 0x75
'byobs',    # 0x76
'byos',    # 0x77
'byoss',    # 0x78
'byong',    # 0x79
'byoj',    # 0x7a
'byoc',    # 0x7b
'byok',    # 0x7c
'byot',    # 0x7d
'byop',    # 0x7e
'byoh',    # 0x7f
'bu',    # 0x80
'bug',    # 0x81
'bugg',    # 0x82
'bugs',    # 0x83
'bun',    # 0x84
'bunj',    # 0x85
'bunh',    # 0x86
'bud',    # 0x87
'bul',    # 0x88
'bulg',    # 0x89
'bulm',    # 0x8a
'bulb',    # 0x8b
'buls',    # 0x8c
'bult',    # 0x8d
'bulp',    # 0x8e
'bulh',    # 0x8f
'bum',    # 0x90
'bub',    # 0x91
'bubs',    # 0x92
'bus',    # 0x93
'buss',    # 0x94
'bung',    # 0x95
'buj',    # 0x96
'buc',    # 0x97
'buk',    # 0x98
'but',    # 0x99
'bup',    # 0x9a
'buh',    # 0x9b
'bweo',    # 0x9c
'bweog',    # 0x9d
'bweogg',    # 0x9e
'bweogs',    # 0x9f
'bweon',    # 0xa0
'bweonj',    # 0xa1
'bweonh',    # 0xa2
'bweod',    # 0xa3
'bweol',    # 0xa4
'bweolg',    # 0xa5
'bweolm',    # 0xa6
'bweolb',    # 0xa7
'bweols',    # 0xa8
'bweolt',    # 0xa9
'bweolp',    # 0xaa
'bweolh',    # 0xab
'bweom',    # 0xac
'bweob',    # 0xad
'bweobs',    # 0xae
'bweos',    # 0xaf
'bweoss',    # 0xb0
'bweong',    # 0xb1
'bweoj',    # 0xb2
'bweoc',    # 0xb3
'bweok',    # 0xb4
'bweot',    # 0xb5
'bweop',    # 0xb6
'bweoh',    # 0xb7
'bwe',    # 0xb8
'bweg',    # 0xb9
'bwegg',    # 0xba
'bwegs',    # 0xbb
'bwen',    # 0xbc
'bwenj',    # 0xbd
'bwenh',    # 0xbe
'bwed',    # 0xbf
'bwel',    # 0xc0
'bwelg',    # 0xc1
'bwelm',    # 0xc2
'bwelb',    # 0xc3
'bwels',    # 0xc4
'bwelt',    # 0xc5
'bwelp',    # 0xc6
'bwelh',    # 0xc7
'bwem',    # 0xc8
'bweb',    # 0xc9
'bwebs',    # 0xca
'bwes',    # 0xcb
'bwess',    # 0xcc
'bweng',    # 0xcd
'bwej',    # 0xce
'bwec',    # 0xcf
'bwek',    # 0xd0
'bwet',    # 0xd1
'bwep',    # 0xd2
'bweh',    # 0xd3
'bwi',    # 0xd4
'bwig',    # 0xd5
'bwigg',    # 0xd6
'bwigs',    # 0xd7
'bwin',    # 0xd8
'bwinj',    # 0xd9
'bwinh',    # 0xda
'bwid',    # 0xdb
'bwil',    # 0xdc
'bwilg',    # 0xdd
'bwilm',    # 0xde
'bwilb',    # 0xdf
'bwils',    # 0xe0
'bwilt',    # 0xe1
'bwilp',    # 0xe2
'bwilh',    # 0xe3
'bwim',    # 0xe4
'bwib',    # 0xe5
'bwibs',    # 0xe6
'bwis',    # 0xe7
'bwiss',    # 0xe8
'bwing',    # 0xe9
'bwij',    # 0xea
'bwic',    # 0xeb
'bwik',    # 0xec
'bwit',    # 0xed
'bwip',    # 0xee
'bwih',    # 0xef
'byu',    # 0xf0
'byug',    # 0xf1
'byugg',    # 0xf2
'byugs',    # 0xf3
'byun',    # 0xf4
'byunj',    # 0xf5
'byunh',    # 0xf6
'byud',    # 0xf7
'byul',    # 0xf8
'byulg',    # 0xf9
'byulm',    # 0xfa
'byulb',    # 0xfb
'byuls',    # 0xfc
'byult',    # 0xfd
'byulp',    # 0xfe
'byulh',    # 0xff
)



# -*- coding: utf-8 -*-

from logging import Logger

import sysactions
from msgbus import MBEasyContext

from mbcontextmessagehandler import MbContextMessageBus
from messagebus import (
    Event,
    Message,
)
from messagehandler import (
    MessageHandler,
    MessageHandlerBuilder,
)
from messageprocessor import (
    DefaultMessageProcessorExecutorFactory,
    MessageProcessorMessageHandler,
)
from messageprocessorutil import LoggingProcessorCallback
from services.manager import DatabaseManager
from typing import Any

from services.interactor import (
    DiscountAppliedInteractor,
    GetConfiguredAlertsInteractor,
    OrderCanceledInteractor,
    POSAlertNotificationInteractor,
    BOHChangeNotificationConfigInteractor,
    BOHOrderPictureInteractor,
    UpdateAlertsInteractor,
)
from services.model import (
    Configs,
    ListenedEvents,
    ListenedTokens,
)
from externals.processor import (
    DiscountAppliedProcessor,
    GetConfiguredAlertsProcessor,
    OrderCanceledProcessor,
    POSAlertNotificationProcessor,
    BOHChangeNotificationConfigProcessor,
    BOHOrderPictureProcessor,
    UpdateAlertsProcessor,
)
from externals.repository import AlertsRepository
from externals.repository.api import AlertsAPIRepository
from services.services import (
    RetrySentEventService,
    UpdateService,
)


class AlertsMessageHandlerBuilder(MessageHandlerBuilder):

    def __init__(
        self,
        configs,            # type: Configs
        message_bus,        # type: MbContextMessageBus
        mb_context,         # type: MBEasyContext
        logger,             # type: Logger
        input_logger,       # type: Logger
        callback_logger,    # type: Logger
    ):
        # type: (...) -> None

        self.configs = configs
        self.message_bus = message_bus
        self.mb_context = mb_context
        self.logger = logger
        self.input_logger = input_logger
        self.callback_logger = callback_logger
        self.cache_manager = None
        sysactions.mbcontext = message_bus.mbcontext
        self.alerts_repository = None
        self.alerts_api_repository = None
        self.update_service = None
        self.database_manager = None

    def build_singletons(
        self,
    ):
        # type: (...) -> None

        self.alerts_repository = AlertsRepository(
            mb_context=self.mb_context,
            configs=self.configs,
            logger=self.logger,
        )
        self.alerts_api_repository = AlertsAPIRepository(
            configs=self.configs,
            logger=self.logger,
        )
        self.update_service = UpdateService(
            message_bus=self.message_bus,
            mb_context=self.mb_context,
            alerts_repository=self.alerts_repository,
            alerts_api_repository=self.alerts_api_repository,
            logger=self.logger,
        )
        retry_sent_event_service = RetrySentEventService(
            message_bus=self.message_bus,
            alerts_repository=self.alerts_repository,
            time_to_call=self.configs.alerts.time_calling,
            logger=self.logger,
        )
        retry_sent_event_service.start()

        self.database_manager = DatabaseManager(
            message_bus=self.message_bus,
            alerts_repository=self.alerts_repository,
            configs=self.configs,
            logger=self.logger,
        )
        self.database_manager.start()

    def build_message_handler(
        self,
        message,    # type: Message
        event,      # type: Event
    ):
        # type: (...) -> MessageHandler

        discount_applied_processor = DiscountAppliedProcessor(
            discount_applied_interactor=DiscountAppliedInteractor(
                alerts_repository=self.alerts_repository,
            )
        )
        get_configured_alerts_processor = GetConfiguredAlertsProcessor(
            get_configured_alerts_interactor=GetConfiguredAlertsInteractor(
                alerts_repository=self.alerts_repository
            )
        )
        order_canceled_processor = OrderCanceledProcessor(
            order_canceled_interactor=OrderCanceledInteractor(
                alerts_repository=self.alerts_repository,
            )
        )
        pos_alert_notification_processor = POSAlertNotificationProcessor(
            pos_alert_notification_interactor=POSAlertNotificationInteractor(
                alerts_repository=self.alerts_repository,
            )
        )
        boh_change_notification_config_processor = BOHChangeNotificationConfigProcessor(
            boh_change_notification_config_interactor=BOHChangeNotificationConfigInteractor(
                update_service=self.update_service,
                logger=self.logger,
            )
        )
        boh_order_picture_processor = BOHOrderPictureProcessor(
            boh_order_picture_interactor=BOHOrderPictureInteractor(
                message_bus=self.message_bus,
                logger=self.logger,
            )
        )
        update_alerts_processor = UpdateAlertsProcessor(
            update_alerts_interactor=UpdateAlertsInteractor(
                update_service=self.update_service,
            )
        )

        message_handler = MessageProcessorMessageHandler(
            event_processors={
                ListenedEvents.DISCOUNT_APPLIED: discount_applied_processor,
                ListenedEvents.ORDER_CANCELED: order_canceled_processor,
                ListenedEvents.POS_ALERT_NOTIFICATION_ACK: pos_alert_notification_processor,
                ListenedEvents.BOH_CHANGE_NOTIFICATION_CONFIG: boh_change_notification_config_processor,
                ListenedEvents.BOH_ORDER_PICTURE: boh_order_picture_processor,
            },
            message_processors={
                ListenedTokens.TK_ALERTS_GET_CONFIGURED_ALERTS.value: get_configured_alerts_processor,
                ListenedTokens.TK_ALERTS_UPDATE_ALERTS.value: update_alerts_processor,
            },
            executor_factory=DefaultMessageProcessorExecutorFactory(
                [
                    LoggingProcessorCallback(self.callback_logger)
                ]
            ),
        )

        return message_handler

    def destroy_singletons(
        self,
    ):
        # type: (...) -> None

        pass

    def destroy_message_handler(
        self,               # type: AlertsMessageHandlerBuilder
        message_handler,    # type: Any
    ):
        # type: (...) -> None

        self.database_manager.stop()



data = (
'hyeo',    # 0x00
'hyeog',    # 0x01
'hyeogg',    # 0x02
'hyeogs',    # 0x03
'hyeon',    # 0x04
'hyeonj',    # 0x05
'hyeonh',    # 0x06
'hyeod',    # 0x07
'hyeol',    # 0x08
'hyeolg',    # 0x09
'hyeolm',    # 0x0a
'hyeolb',    # 0x0b
'hyeols',    # 0x0c
'hyeolt',    # 0x0d
'hyeolp',    # 0x0e
'hyeolh',    # 0x0f
'hyeom',    # 0x10
'hyeob',    # 0x11
'hyeobs',    # 0x12
'hyeos',    # 0x13
'hyeoss',    # 0x14
'hyeong',    # 0x15
'hyeoj',    # 0x16
'hyeoc',    # 0x17
'hyeok',    # 0x18
'hyeot',    # 0x19
'hyeop',    # 0x1a
'hyeoh',    # 0x1b
'hye',    # 0x1c
'hyeg',    # 0x1d
'hyegg',    # 0x1e
'hyegs',    # 0x1f
'hyen',    # 0x20
'hyenj',    # 0x21
'hyenh',    # 0x22
'hyed',    # 0x23
'hyel',    # 0x24
'hyelg',    # 0x25
'hyelm',    # 0x26
'hyelb',    # 0x27
'hyels',    # 0x28
'hyelt',    # 0x29
'hyelp',    # 0x2a
'hyelh',    # 0x2b
'hyem',    # 0x2c
'hyeb',    # 0x2d
'hyebs',    # 0x2e
'hyes',    # 0x2f
'hyess',    # 0x30
'hyeng',    # 0x31
'hyej',    # 0x32
'hyec',    # 0x33
'hyek',    # 0x34
'hyet',    # 0x35
'hyep',    # 0x36
'hyeh',    # 0x37
'ho',    # 0x38
'hog',    # 0x39
'hogg',    # 0x3a
'hogs',    # 0x3b
'hon',    # 0x3c
'honj',    # 0x3d
'honh',    # 0x3e
'hod',    # 0x3f
'hol',    # 0x40
'holg',    # 0x41
'holm',    # 0x42
'holb',    # 0x43
'hols',    # 0x44
'holt',    # 0x45
'holp',    # 0x46
'holh',    # 0x47
'hom',    # 0x48
'hob',    # 0x49
'hobs',    # 0x4a
'hos',    # 0x4b
'hoss',    # 0x4c
'hong',    # 0x4d
'hoj',    # 0x4e
'hoc',    # 0x4f
'hok',    # 0x50
'hot',    # 0x51
'hop',    # 0x52
'hoh',    # 0x53
'hwa',    # 0x54
'hwag',    # 0x55
'hwagg',    # 0x56
'hwags',    # 0x57
'hwan',    # 0x58
'hwanj',    # 0x59
'hwanh',    # 0x5a
'hwad',    # 0x5b
'hwal',    # 0x5c
'hwalg',    # 0x5d
'hwalm',    # 0x5e
'hwalb',    # 0x5f
'hwals',    # 0x60
'hwalt',    # 0x61
'hwalp',    # 0x62
'hwalh',    # 0x63
'hwam',    # 0x64
'hwab',    # 0x65
'hwabs',    # 0x66
'hwas',    # 0x67
'hwass',    # 0x68
'hwang',    # 0x69
'hwaj',    # 0x6a
'hwac',    # 0x6b
'hwak',    # 0x6c
'hwat',    # 0x6d
'hwap',    # 0x6e
'hwah',    # 0x6f
'hwae',    # 0x70
'hwaeg',    # 0x71
'hwaegg',    # 0x72
'hwaegs',    # 0x73
'hwaen',    # 0x74
'hwaenj',    # 0x75
'hwaenh',    # 0x76
'hwaed',    # 0x77
'hwael',    # 0x78
'hwaelg',    # 0x79
'hwaelm',    # 0x7a
'hwaelb',    # 0x7b
'hwaels',    # 0x7c
'hwaelt',    # 0x7d
'hwaelp',    # 0x7e
'hwaelh',    # 0x7f
'hwaem',    # 0x80
'hwaeb',    # 0x81
'hwaebs',    # 0x82
'hwaes',    # 0x83
'hwaess',    # 0x84
'hwaeng',    # 0x85
'hwaej',    # 0x86
'hwaec',    # 0x87
'hwaek',    # 0x88
'hwaet',    # 0x89
'hwaep',    # 0x8a
'hwaeh',    # 0x8b
'hoe',    # 0x8c
'hoeg',    # 0x8d
'hoegg',    # 0x8e
'hoegs',    # 0x8f
'hoen',    # 0x90
'hoenj',    # 0x91
'hoenh',    # 0x92
'hoed',    # 0x93
'hoel',    # 0x94
'hoelg',    # 0x95
'hoelm',    # 0x96
'hoelb',    # 0x97
'hoels',    # 0x98
'hoelt',    # 0x99
'hoelp',    # 0x9a
'hoelh',    # 0x9b
'hoem',    # 0x9c
'hoeb',    # 0x9d
'hoebs',    # 0x9e
'hoes',    # 0x9f
'hoess',    # 0xa0
'hoeng',    # 0xa1
'hoej',    # 0xa2
'hoec',    # 0xa3
'hoek',    # 0xa4
'hoet',    # 0xa5
'hoep',    # 0xa6
'hoeh',    # 0xa7
'hyo',    # 0xa8
'hyog',    # 0xa9
'hyogg',    # 0xaa
'hyogs',    # 0xab
'hyon',    # 0xac
'hyonj',    # 0xad
'hyonh',    # 0xae
'hyod',    # 0xaf
'hyol',    # 0xb0
'hyolg',    # 0xb1
'hyolm',    # 0xb2
'hyolb',    # 0xb3
'hyols',    # 0xb4
'hyolt',    # 0xb5
'hyolp',    # 0xb6
'hyolh',    # 0xb7
'hyom',    # 0xb8
'hyob',    # 0xb9
'hyobs',    # 0xba
'hyos',    # 0xbb
'hyoss',    # 0xbc
'hyong',    # 0xbd
'hyoj',    # 0xbe
'hyoc',    # 0xbf
'hyok',    # 0xc0
'hyot',    # 0xc1
'hyop',    # 0xc2
'hyoh',    # 0xc3
'hu',    # 0xc4
'hug',    # 0xc5
'hugg',    # 0xc6
'hugs',    # 0xc7
'hun',    # 0xc8
'hunj',    # 0xc9
'hunh',    # 0xca
'hud',    # 0xcb
'hul',    # 0xcc
'hulg',    # 0xcd
'hulm',    # 0xce
'hulb',    # 0xcf
'huls',    # 0xd0
'hult',    # 0xd1
'hulp',    # 0xd2
'hulh',    # 0xd3
'hum',    # 0xd4
'hub',    # 0xd5
'hubs',    # 0xd6
'hus',    # 0xd7
'huss',    # 0xd8
'hung',    # 0xd9
'huj',    # 0xda
'huc',    # 0xdb
'huk',    # 0xdc
'hut',    # 0xdd
'hup',    # 0xde
'huh',    # 0xdf
'hweo',    # 0xe0
'hweog',    # 0xe1
'hweogg',    # 0xe2
'hweogs',    # 0xe3
'hweon',    # 0xe4
'hweonj',    # 0xe5
'hweonh',    # 0xe6
'hweod',    # 0xe7
'hweol',    # 0xe8
'hweolg',    # 0xe9
'hweolm',    # 0xea
'hweolb',    # 0xeb
'hweols',    # 0xec
'hweolt',    # 0xed
'hweolp',    # 0xee
'hweolh',    # 0xef
'hweom',    # 0xf0
'hweob',    # 0xf1
'hweobs',    # 0xf2
'hweos',    # 0xf3
'hweoss',    # 0xf4
'hweong',    # 0xf5
'hweoj',    # 0xf6
'hweoc',    # 0xf7
'hweok',    # 0xf8
'hweot',    # 0xf9
'hweop',    # 0xfa
'hweoh',    # 0xfb
'hwe',    # 0xfc
'hweg',    # 0xfd
'hwegg',    # 0xfe
'hwegs',    # 0xff
)



from typing import Any

from interactor import BaseInteractor
from messagebus import Message, DefaultToken
from messageprocessorutil import UuidMessageProcessor
from model.log import LogEntry


class EventProcessor(UuidMessageProcessor):
    def __init__(self, interactor, is_message=False, dummy=False):
        # type: (BaseInteractor, bool, bool) -> None
        super(EventProcessor, self).__init__()
        self.interactor = interactor
        self.is_message = is_message
        self.dummy = dummy

    def parse_data(self, data, event_type):
        # type: (str, str) -> Any

        return data

    def call_business(self, log_entry):
        # type: (LogEntry) -> None
        if log_entry is not None and not self.dummy:
            self.interactor.execute(log_entry)

    def format_response(self, message_bus, message, event, input_obj, result):
        if self.is_message:
            message_bus.reply_message(
                message=message,
                reply_message=Message(DefaultToken.TK_SYS_ACK),
            )
        elif event.synchronous:
            message_bus.reply_event(
                event=event,
                reply_message=Message(DefaultToken.TK_SYS_NAK),
            )

    def format_exception(self, message_bus, message, event, input_obj, exception):
        if self.is_message:
            message_bus.reply_message(message, Message(DefaultToken.TK_SYS_NAK))
        elif event.synchronous:
            message_bus.reply_event(event, Message(DefaultToken.TK_SYS_NAK))

    def format_parse_exception(self, message_bus, message, event, data, exception):
        self.format_exception(message_bus, message, event, data, exception)




from ScaleEvent import ScaleEvent

class PosScaleEvents(object):
    def __init__(self):
        self._wait_pos_id_event_dict = {}
        self._service_name_dict = {}
        self._scale_event_dict = {}

    def create_scale_event(self, pos_id, service_name):
        scale_event = ScaleEvent()
        self._wait_pos_id_event_dict[pos_id] = scale_event
        self._service_name_dict[service_name] = scale_event
        self._scale_event_dict[scale_event] = (pos_id, service_name)

        return scale_event

    def add_weight(self, service_name, weight):
        if service_name not in self._service_name_dict:
            return

        self._service_name_dict[service_name].set_weight(weight)

    def finish_event(self, scale_event):
        pos_id, service_name = self._scale_event_dict[scale_event]

        del self._wait_pos_id_event_dict[pos_id]
        del self._service_name_dict[service_name]
        del self._scale_event_dict[scale_event]


    def get_wait_pos_id_event_dict(self):
        return self._wait_pos_id_event_dict


import logging.handlers
import os
from logging import (
    LogRecord,
    Formatter,
)

from sysactions import action

LOGGER_NAME = "ExternalLog_{}.log"
FIVE_MEGABYTES = 5242880

configured_handlers = {}


class NoNewLineFormatter(Formatter):
    def format(
        self,       # type: NoNewLineFormatter
        record,     # type: LogRecord
    ):
        ret = super(NoNewLineFormatter, self).format(record)
        if ret[-1] == '\n':
            return ret[:-1]
        return ret


@action
def external_log(
    pos_id,         # type: str
    message,        # type: str
):
    if pos_id not in configured_handlers:
        _add_rotating_file_handler_to_pos_id(pos_id)

    rotating_file_handler = configured_handlers[pos_id]
    rotating_file_handler.emit(
        _create_dummy_log_record(message=message)
    )


def _add_rotating_file_handler_to_pos_id(
    pos_id,     # type: str
):
    pos_logger_name = LOGGER_NAME.format(pos_id.zfill(2))
    pyscripts_directory = os.path.abspath(os.path.dirname(os.environ["LOADERCFG"]))
    file_name = os.path.join(pyscripts_directory, pos_logger_name)
    handler = logging.handlers.RotatingFileHandler(
        filename=file_name,
        mode='a',
        maxBytes=FIVE_MEGABYTES,
        backupCount=5,
    )
    handler.setFormatter(NoNewLineFormatter())
    configured_handlers[pos_id] = handler


def _create_dummy_log_record(
    message,        # type: str
):
    return LogRecord(
        name="",
        level=logging.DEBUG,
        pathname="",
        lineno=0,
        msg=message,
        args=(),
        exc_info=None,
    )



from datetime import datetime

from typing import Optional, Union

from msgbus import MBEasyContext

from application.model import UpdateControl
from typing import Optional

update_control = UpdateControl()
settings_mb_context = None


def init(
        mb_context  # type: MBEasyContext
):
    # type: (...) -> None

    global settings_mb_context
    settings_mb_context = mb_context


def set_update_control(key, value):
    # type: (str, Optional[Union[str, bool]]) -> None

    global update_control
    update_control.set_status(key, value)

    if key == "updating" and value is True:
        show_update_version_screen(settings_mb_context)


def check_status(pos_version):

    global update_control
    update_control.check_status(pos_version)


def clear_status():

    global update_control
    update_control.clear_all_status()


def show_update_version_screen(
        mb_context  # type: Optional[MBEasyContext]
):
    # type: (...) -> None

    data = str(datetime.now())
    mb_context.MB_EasyEvtSend("FETCH_POS_VERSIONS", "", data)



# coding=utf-8
import base64

from modelcommon.electronicpayment import TransactionProcessors
from modelcommon.model import SitefResponse
from typing import (
    Any,
    Union,
    Optional,
)

from model import (
    CancelPaymentOrderData,
    ConfirmFinishPaymentData,
    PaymentResponseData,
)
from paymentprocessor import PaymentProcessor
from util import get_transaction_identifier


class FakePaymentProcessor(PaymentProcessor):
    def __init__(
        self,
        message_bus,        # type: MbContextMessageBus
        dialog_manager,     # type: DialogManager
        config,             # type: Group
        logger,             # type: Logger
        cache,              # type: Cache
    ):
        # type: (...) -> None

        super(FakePaymentProcessor, self).__init__(
            message_bus,
            dialog_manager,
            config,
            logger,
            cache,
        )

        self.pdv_tef = int(config.find_value("PdvTef") or config.find_value("PosId"))
        self.pos_id = int(config.find_value("PosId"))
        self.cache = cache

    def get_last_receipt(
        self,
        eft_processor,      # type: str
    ):
        # type: (...) -> str

        return self.cache.get_last_receipt(eft_processor=eft_processor)

    def connection_status(self, test_connection_data):
        return True

    def add_payment(self, start_payment_data):
        pos_id = start_payment_data.pos_id
        transaction_identifier = get_transaction_identifier(
            order_id=start_payment_data.order_id,
            tender_id=start_payment_data.electronic_type
        )

        card_brand = {
            1: "01",
            2: "02",
        }.get(start_payment_data.electronic_type, "99")

        self.dialog_manager.close_dialogs(pos_id)
        return PaymentResponseData(
            media="",
            result="0",
            card_number="4111XXXXXXXXXXXX",
            auth_code="1302",
            approved_amount=start_payment_data.amount,
            id_auth="08496898000142",
            cnpj_auth="68536379000192",
            bandeira=card_brand,
            receipt_merchant=base64.b64encode("Recibo de teste.\n MERCHANT\nSitef FAKE"),
            receipt_customer=base64.b64encode("Recibo de teste.\n CUSTOMER\nSitef FAKE"),
            expiration_date="05/2029",
            nsu="1234",
            transaction_processor=TransactionProcessors.FAKE,
            last_digits="1111",
            sat_acquirer="012",
            t_pag="03",
            t_band="01",
            nsu_host="4321",
            electronic_type=start_payment_data.electronic_type,
            transaction_identifier=transaction_identifier
        )

    def cancel_payment_order(
        self,
        cancel_payment_order_data,  # type: CancelPaymentOrderData
        additional_params="",       # type: Optional[str]
    ):
        # type: (...) -> Any[Union[SitefResponse, str]]

        response = SitefResponse()
        response.nsu = "123"
        response.nsu_host = "321"
        response.nsu_origin = "555"
        response.transaction_value = "10"
        response.customer_receipt = base64.b64encode("Recibo de teste CANCELAMENTO.\n CUSTOMER\nSitef FAKE")
        response.merchant_receipt = base64.b64encode("Recibo de teste CANCELAMENTO.\n MERCHANT\nSitef FAKE")
        response.transaction_identifier = get_transaction_identifier(
            order_id=cancel_payment_order_data.order_id,
            tender_id=cancel_payment_order_data.tender_id
        )
        return response

    def cancel_payment(self):
        return ""

    def finish_payment(self, finish_payment_data):
        return True

    def confirm_finish_payment(
        self,
        confirm_finish_payment_data     # type: ConfirmFinishPaymentData
    ):
        # type: (...) -> None

        pass

    def get_name(
        self,
    ):
        # type: (...) -> str

        self.register_eft_processor(
            pos_id=self.pos_id,
            processor_name=TransactionProcessors.FAKE,
        )

        return TransactionProcessors.FAKE

    def get_document(
        self,
        pos_id      # type: str
    ):
        # type: (...) -> str

        return "68536379000192"



from ..datastructures import CharsetAccept
from ..datastructures import LanguageAccept
from ..datastructures import MIMEAccept
from ..http import parse_accept_header
from ..utils import cached_property


class AcceptMixin(object):
    """A mixin for classes with an :attr:`~BaseResponse.environ` attribute
    to get all the HTTP accept headers as
    :class:`~werkzeug.datastructures.Accept` objects (or subclasses
    thereof).
    """

    @cached_property
    def accept_mimetypes(self):
        """List of mimetypes this client supports as
        :class:`~werkzeug.datastructures.MIMEAccept` object.
        """
        return parse_accept_header(self.environ.get("HTTP_ACCEPT"), MIMEAccept)

    @cached_property
    def accept_charsets(self):
        """List of charsets this client supports as
        :class:`~werkzeug.datastructures.CharsetAccept` object.
        """
        return parse_accept_header(
            self.environ.get("HTTP_ACCEPT_CHARSET"), CharsetAccept
        )

    @cached_property
    def accept_encodings(self):
        """List of encodings this client accepts.  Encodings in a HTTP term
        are compression encodings such as gzip.  For charsets have a look at
        :attr:`accept_charset`.
        """
        return parse_accept_header(self.environ.get("HTTP_ACCEPT_ENCODING"))

    @cached_property
    def accept_languages(self):
        """List of languages this client accepts as
        :class:`~werkzeug.datastructures.LanguageAccept` object.

        .. versionchanged 0.5
           In previous versions this was a regular
           :class:`~werkzeug.datastructures.Accept` object.
        """
        return parse_accept_header(
            self.environ.get("HTTP_ACCEPT_LANGUAGE"), LanguageAccept
        )



from datetime import datetime

from application.interactor import UserUpdaterInteractor
from application.model import UpdateType
from application.model.configuration import Configurations
from application.model.customexception import NoUpdateFound
from application.repository.pos import (
    UserUpdaterPOSRepository,
    BinaryUpdaterPOSRepository,
)
from application.util import LoggerUtil

from ._UpdaterManager import UpdaterManager

logger = LoggerUtil.get_logger_name(__name__)


class UserUpdaterManager(UpdaterManager):
    def __init__(
        self,
        configs,                # type: Configurations
        interactor,             # type: UserUpdaterInteractor
        pos_repository,         # type: UserUpdaterPOSRepository
        binaries_repository,    # type: BinaryUpdaterPOSRepository
    ):
        # type: (...) -> None

        name = UpdateType.user.name
        super(UserUpdaterManager, self).__init__(name, configs, interactor, pos_repository, logger)
        self.logger = logger
        self.interactor = interactor
        self.binaries_repository = binaries_repository

    def update(self):
        # type: () -> None

        self.interactor.update_users([])

    def has_update_to_apply(self):
        # type: () -> None

        if not self.binaries_repository.has_already_applied_update():
            raise NoUpdateFound()

        last_update_date = self.pos_repository.get_last_user_update_date()
        today = datetime.today().replace(hour=0, minute=0, second=0, microsecond=0)
        update_to_apply = last_update_date is None or last_update_date < today
        if not update_to_apply:
            raise NoUpdateFound()



# bankaccount.py - functions for handling Czech bank account numbers
# coding: utf-8
#
# Copyright (C) 2022 Petr Přikryl
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301 USA

"""Czech bank account number.

The Czech bank account numbers consist of up to 20 digits:
    UUUUUK-MMMMMMMMKM/XXXX

The first part is prefix that is up to 6 digits. The following part is from 2 to 10 digits.
Both parts could be filled with zeros from left if missing.
The final 4 digits represent the bank code.

More information:

* https://www.penize.cz/osobni-ucty/424173-tajemstvi-cisla-uctu-klicem-pro-banky-je-11
* https://www.zlatakoruna.info/zpravy/ucty/cislo-uctu-v-cr

>>> validate('34278-0727558021/0100')
'034278-0727558021/0100'
>>> validate('4278-727558021/0100')  # invalid check digits (prefix)
Traceback (most recent call last):
    ...
InvalidChecksum: ...
>>> validate('34278-727558021/0000')  # invalid bank
Traceback (most recent call last):
    ...
InvalidComponent: ...
>>> format('34278-727558021/0100')
'034278-0727558021/0100'
>>> to_bic('34278-727558021/0100')
'KOMBCZPP'
"""

import re

from stdnum.exceptions import *
from stdnum.util import clean


_bankaccount_re = re.compile(
    r'((?P<prefix>[0-9]{0,6})-)?(?P<root>[0-9]{2,10})\/(?P<bank>[0-9]{4})')


def compact(number):
    """Convert the number to the minimal representation. This strips the
    number of any valid separators and removes surrounding whitespace."""
    number = clean(number).strip()
    match = _bankaccount_re.match(number)
    if match:
        # zero-pad valid numbers
        prefix = (match.group('prefix') or '').zfill(6)
        root = match.group('root').zfill(10)
        number = ''.join((prefix, '-', root, '/', match.group('bank')))
    return number


def _split(number):
    """Split valid numbers into prefix, root and bank parts of the number."""
    match = _bankaccount_re.match(number)
    if not match:
        raise InvalidFormat()
    return match.group('prefix'), match.group('root'), match.group('bank')


def _info(bank):
    """Look up information for the bank."""
    from stdnum import numdb
    info = {}
    for _nr, found in numdb.get('cz/banks').info(bank):
        info.update(found)
    return info


def info(number):
    """Return a dictionary of data about the supplied number. This typically
    returns the name of the bank and branch and a BIC if it is valid."""
    prefix, root, bank = _split(compact(number))
    return _info(bank)


def to_bic(number):
    """Return the BIC for the bank that this number refers to."""
    bic = info(number).get('bic')
    if bic:
        return str(bic)


def _calc_checksum(number):
    weights = (6, 3, 7, 9, 10, 5, 8, 4, 2, 1)
    return sum(w * int(n) for w, n in zip(weights, number.zfill(10))) % 11


def validate(number):
    """Check if the number provided is a valid bank account number."""
    number = compact(number)
    prefix, root, bank = _split(number)
    if _calc_checksum(prefix) != 0:
        raise InvalidChecksum()
    if _calc_checksum(root) != 0:
        raise InvalidChecksum()
    if 'bank' not in _info(bank):
        raise InvalidComponent()
    return number


def is_valid(number):
    """Check if the number provided is a valid bank account number."""
    try:
        return bool(validate(number))
    except ValidationError:
        return False


def format(number):
    """Reformat the number to the standard presentation format."""
    return compact(number)
